@@ -17,8 +17,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-import sys
-from mutagen import _util
 from picard import log
 from picard.plugin import ExtensionPoint
 
@@ -26,15 +24,15 @@ _formats = ExtensionPoint()
 _extensions = {}
 
 
-def register_format(format):
-    _formats.register(format.__module__, format)
-    for ext in format.EXTENSIONS:
-        _extensions[ext[1:]] = format
+def register_format(file_format):
+    _formats.register(file_format.__module__, file_format)
+    for ext in file_format.EXTENSIONS:
+        _extensions[ext[1:]] = file_format
 
 
 def supported_formats():
     """Returns list of supported formats."""
-    return [(format.EXTENSIONS, format.NAME) for format in _formats]
+    return [(file_format.EXTENSIONS, file_format.NAME) for file_format in _formats]
 
 
 def supported_extensions():
@@ -42,12 +40,16 @@ def supported_extensions():
     return [ext for exts, name in supported_formats() for ext in exts]
 
 
+def ext_to_format(ext):
+    return _extensions.get(ext, None)
+
+
 def guess_format(filename, options=_formats):
     """Select the best matching file type amongst supported formats."""
     results = []
     # Since we are reading only 128 bytes and then immediately closing the file,
     # use unbuffered mode.
-    with file(filename, "rb", 0) as fileobj:
+    with open(filename, "rb", 0) as fileobj:
         header = fileobj.read(128)
         # Calls the score method of a particular format's associated filetype
         # and assigns a positive score depending on how closely the fileobj's header matches
@@ -65,7 +67,7 @@ def guess_format(filename, options=_formats):
     return None
 
 
-def open(filename):
+def open_(filename):
     """Open the specified file and return a File instance with the appropriate format handler, or None."""
     try:
         # First try to guess the format on the basis of file headers
@@ -82,17 +84,18 @@ def open(filename):
         # None is returned if both the methods fail
         return None
     except Exception as error:
-        log.error("Error occured:\n{}".format(error.message))
+        log.error("Error occurred:\n{}".format(error))
         return None
 
 
 from picard.formats.id3 import (
     AiffFile,
+    DSFFile,
     MP3File,
     TrueAudioFile,
 )
-if AiffFile:
-    register_format(AiffFile)
+register_format(AiffFile)
+register_format(DSFFile)
 register_format(MP3File)
 register_format(TrueAudioFile)
 
@@ -117,28 +120,26 @@ from picard.formats.vorbis import (
     OggAudioFile,
     OggVideoFile,
     OggOpusFile,
-    with_opus,
 )
 register_format(FLACFile)
 register_format(OggFLACFile)
 register_format(OggSpeexFile)
 register_format(OggVorbisFile)
-if with_opus:
-    register_format(OggOpusFile)
+register_format(OggOpusFile)
 register_format(OggAudioFile)
 register_format(OggVideoFile)
 
-try:
-    from picard.formats.mp4 import MP4File
-    register_format(MP4File)
-except ImportError:
-    pass
+from picard.formats.mp4 import MP4File
+register_format(MP4File)
 
-try:
-    from picard.formats.asf import ASFFile
-    register_format(ASFFile)
-except ImportError:
-    pass
+from picard.formats.asf import ASFFile
+register_format(ASFFile)
 
 from picard.formats.wav import WAVFile
 register_format(WAVFile)
+
+from picard.formats.midi import MIDIFile
+register_format(MIDIFile)
+
+from picard.formats.aac import AACFile
+register_format(AACFile)
