@@ -19,21 +19,35 @@
 
 import base64
 import re
+
 import mutagen.flac
 import mutagen.ogg
 import mutagen.oggflac
+import mutagen.oggopus
 import mutagen.oggspeex
 import mutagen.oggtheora
 import mutagen.oggvorbis
-import mutagen.oggopus
 
-from picard import config, log
-from picard.coverart.image import TagCoverArtImage, CoverArtImageError
+from picard import (
+    config,
+    log,
+)
+from picard.coverart.image import (
+    CoverArtImageError,
+    TagCoverArtImage,
+)
 from picard.file import File
-from picard.formats.id3 import types_from_id3, image_type_as_id3_num
-from picard.metadata import Metadata
-from picard.util import encode_filename, sanitize_date
 from picard.formats import guess_format
+from picard.formats.id3 import (
+    image_type_as_id3_num,
+    types_from_id3,
+)
+from picard.metadata import Metadata
+from picard.util import (
+    encode_filename,
+    sanitize_date,
+)
+
 
 class VCommentFile(File):
 
@@ -41,8 +55,10 @@ class VCommentFile(File):
     _File = None
 
     __translate = {
-        "musicbrainz_trackid": "musicbrainz_recordingid",
+        "movement": "movementnumber",
+        "movementname": "movement",
         "musicbrainz_releasetrackid": "musicbrainz_trackid",
+        "musicbrainz_trackid": "musicbrainz_recordingid",
         "waveformatextensible_channel_mask": "~waveformatextensible_channel_mask",
     }
     __rtranslate = dict([(v, k) for k, v in __translate.items()])
@@ -81,7 +97,10 @@ class VCommentFile(File):
                     if email != config.setting['rating_user_email']:
                         continue
                     name = '~rating'
-                    value = string_(int(round((float(value) * (config.setting['rating_steps'] - 1)))))
+                    try:
+                        value = str(round((float(value) * (config.setting['rating_steps'] - 1))))
+                    except ValueError:
+                        log.warning('Invalid rating value in %r: %s', filename, value)
                 elif name == "fingerprint" and value.startswith("MusicMagic Fingerprint"):
                     name = "musicip_fingerprint"
                     value = value[22:]
@@ -171,9 +190,9 @@ class VCommentFile(File):
                     name = 'rating:%s' % config.setting['rating_user_email']
                 else:
                     name = 'rating'
-                value = string_(float(value) / (config.setting['rating_steps'] - 1))
+                value = str(float(value) / (config.setting['rating_steps'] - 1))
             # don't save private tags
-            elif name.startswith("~"):
+            elif name.startswith("~") or not self.supports_tag(name):
                 continue
             elif name.startswith('lyrics:'):
                 name = 'lyrics'
@@ -262,7 +281,8 @@ class VCommentFile(File):
 
     @classmethod
     def supports_tag(cls, name):
-        return bool(name)
+        unsupported_tags = {}
+        return bool(name) and name not in unsupported_tags
 
 
 class FLACFile(VCommentFile):
@@ -284,10 +304,6 @@ class OggFLACFile(VCommentFile):
     NAME = "Ogg FLAC"
     _File = mutagen.oggflac.OggFLAC
 
-    def _info(self, metadata, file):
-        super()._info(metadata, file)
-        metadata['~format'] = self.NAME
-
 
 class OggSpeexFile(VCommentFile):
 
@@ -296,10 +312,6 @@ class OggSpeexFile(VCommentFile):
     NAME = "Speex"
     _File = mutagen.oggspeex.OggSpeex
 
-    def _info(self, metadata, file):
-        super()._info(metadata, file)
-        metadata['~format'] = self.NAME
-
 
 class OggTheoraFile(VCommentFile):
 
@@ -308,10 +320,6 @@ class OggTheoraFile(VCommentFile):
     NAME = "Ogg Theora"
     _File = mutagen.oggtheora.OggTheora
 
-    def _info(self, metadata, file):
-        super()._info(metadata, file)
-        metadata['~format'] = self.NAME
-
 
 class OggVorbisFile(VCommentFile):
 
@@ -320,10 +328,6 @@ class OggVorbisFile(VCommentFile):
     NAME = "Ogg Vorbis"
     _File = mutagen.oggvorbis.OggVorbis
 
-    def _info(self, metadata, file):
-        super()._info(metadata, file)
-        metadata['~format'] = self.NAME
-
 
 class OggOpusFile(VCommentFile):
 
@@ -332,16 +336,13 @@ class OggOpusFile(VCommentFile):
     NAME = "Ogg Opus"
     _File = mutagen.oggopus.OggOpus
 
-    def _info(self, metadata, file):
-        super()._info(metadata, file)
-        metadata['~format'] = self.NAME
-
 
 def OggAudioFile(filename):
     """Generic Ogg audio file."""
     options = [OggFLACFile, OggSpeexFile, OggVorbisFile]
     return guess_format(filename, options)
 
+
 OggAudioFile.EXTENSIONS = [".oga"]
 OggAudioFile.NAME = "Ogg Audio"
 
