@@ -17,36 +17,40 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-import os
-import sys
 from functools import partial
-from PyQt4 import QtCore, QtGui, QtNetwork
-from picard import config, log
+import os
+
+from PyQt5 import (
+    QtCore,
+    QtGui,
+    QtNetwork,
+    QtWidgets,
+)
+
+from picard import (
+    config,
+    log,
+)
 from picard.album import Album
-from picard.coverart.image import CoverArtImage, CoverArtImageError
-from picard.track import Track
+from picard.const import MAX_COVERS_TO_STACK
+from picard.coverart.image import (
+    CoverArtImage,
+    CoverArtImageError,
+)
 from picard.file import File
+from picard.track import Track
 from picard.util import imageinfo
 from picard.util.lrucache import LRUCache
-from picard.const import MAX_COVERS_TO_STACK
-
-if sys.platform == 'darwin':
-    try:
-        from Foundation import NSURL
-        NSURL_IMPORTED = True
-    except ImportError:
-        NSURL_IMPORTED = False
-        log.warning("Unable to import NSURL, file drag'n'drop might not work correctly")
 
 
-class ActiveLabel(QtGui.QLabel):
+class ActiveLabel(QtWidgets.QLabel):
     """Clickable QLabel."""
 
     clicked = QtCore.pyqtSignal()
-    image_dropped = QtCore.pyqtSignal(QtCore.QUrl, QtCore.QByteArray)
+    image_dropped = QtCore.pyqtSignal(QtCore.QUrl, bytes)
 
     def __init__(self, active=True, drops=False, *args):
-        QtGui.QLabel.__init__(self, *args)
+        super().__init__(*args)
         self.setMargin(0)
         self.setActive(active)
         self.setAcceptDrops(drops)
@@ -62,22 +66,55 @@ class ActiveLabel(QtGui.QLabel):
         if self.active and event.button() == QtCore.Qt.LeftButton:
             self.clicked.emit()
 
-    def dragEnterEvent(self, event):
-        for url in event.mimeData().urls():
-            if url.scheme() in ('https', 'http', 'file'):
-                event.acceptProposedAction()
-                break
+    @staticmethod
+    def dragEnterEvent(event):
+        event.acceptProposedAction()
+
+    @staticmethod
+    def dragMoveEvent(event):
+        event.acceptProposedAction()
 
     def dropEvent(self, event):
         accepted = False
         # Chromium includes the actual data of the dragged image in the drop event. This
         # is useful for Google Images, where the url links to the page that contains the image
         # so we use it if the downloaded url is not an image.
-        dropped_data = event.mimeData().data('application/octet-stream')
-        for url in event.mimeData().urls():
-            if url.scheme() in ('https', 'http', 'file'):
+        mime_data = event.mimeData()
+        dropped_data = bytes(mime_data.data('application/octet-stream'))
+
+        if not dropped_data:
+            dropped_data = bytes(mime_data.data('application/x-qt-image'))
+
+        if dropped_data:
+            try:
+                mime = imageinfo.identify(dropped_data)[2]
+                if mime in ('image/jpeg', 'image/png'):
+                    accepted = True
+                    log.debug("Dropped %s mime data (%d bytes)", mime,
+                              len(dropped_data or ''))
+                    self.image_dropped.emit(QtCore.QUrl(''), dropped_data)
+            except imageinfo.IdentificationError as e:
+                log.debug("Image identification failed: %s", e)
+
+        if not accepted:
+            for url in event.mimeData().urls():
+                if url.scheme() in ('https', 'http', 'file'):
+                    accepted = True
+                    log.debug("Dropped %s url (with %d bytes of data)",
+                              url.toString(), len(dropped_data or ''))
+                    self.image_dropped.emit(url, dropped_data)
+
+        if not accepted:
+            if mime_data.hasImage():
+                image_bytes = QtCore.QByteArray()
+                image_buffer = QtCore.QBuffer(image_bytes)
+                mime_data.imageData().save(image_buffer, 'JPEG')
+                dropped_data = bytes(image_bytes)
+
                 accepted = True
-                self.image_dropped.emit(url, dropped_data)
+                log.debug("Dropped %d bytes of Qt image data", len(dropped_data))
+                self.image_dropped.emit(QtCore.QUrl(''), dropped_data)
+
         if accepted:
             event.acceptProposedAction()
 
@@ -85,10 +122,14 @@ class ActiveLabel(QtGui.QLabel):
 class CoverArtThumbnail(ActiveLabel):
 
     def __init__(self, active=False, drops=False, pixmap_cache=None, *args, **kwargs):
-        super(CoverArtThumbnail, self).__init__(active, drops, *args, **kwargs)
+        super().__init__(active, drops, *args, **kwargs)
         self.data = None
         self.has_common_images = None
         self.shadow = QtGui.QPixmap(":/images/CoverArtShadow.png")
+        self.pixel_ratio = self.tagger.primaryScreen().devicePixelRatio()
+        w, h = self.scaled(128, 128)
+        self.shadow = self.shadow.scaled(w, h, QtCore.Qt.KeepAspectRatio, QtCore.Qt.SmoothTransformation)
+        self.shadow.setDevicePixelRatio(self.pixel_ratio)
         self.release = None
         self.setPixmap(self.shadow)
         self.setAlignment(QtCore.Qt.AlignTop | QtCore.Qt.AlignHCenter)
@@ -103,18 +144,22 @@ class CoverArtThumbnail(ActiveLabel):
         else:
             return True
 
+    def scaled(self, *dimensions):
+        return (self.pixel_ratio * dimension for dimension in dimensions)
+
     def show(self):
         self.set_data(self.data, True)
 
     def decorate_cover(self, pixmap):
-        offx, offy, w, h = (1, 1, 121, 121)
+        offx, offy, w, h = self.scaled(1, 1, 121, 121)
         cover = QtGui.QPixmap(self.shadow)
         pixmap = pixmap.scaled(w, h, QtCore.Qt.KeepAspectRatio, QtCore.Qt.SmoothTransformation)
+        pixmap.setDevicePixelRatio(self.pixel_ratio)
         painter = QtGui.QPainter(cover)
         bgcolor = QtGui.QColor.fromRgb(0, 0, 0, 128)
         painter.fillRect(QtCore.QRectF(offx, offy, w, h), bgcolor)
-        x = offx + (w - pixmap.width()) / 2
-        y = offy + (h - pixmap.height()) / 2
+        x = offx + (w - pixmap.width()) // 2
+        y = offy + (h - pixmap.height()) // 2
         painter.drawPixmap(x, y, pixmap)
         painter.end()
         return cover
@@ -137,8 +182,8 @@ class CoverArtThumbnail(ActiveLabel):
         if len(self.data) == 1:
             has_common_images = True
 
-        w, h, displacements = (128, 128, 20)
-        key = hash(tuple(sorted(self.data)) + (has_common_images,))
+        w, h, displacements = self.scaled(128, 128, 20)
+        key = hash(tuple(sorted(self.data, key=lambda x: x.types_as_string())) + (has_common_images,))
         try:
             pixmap = self._pixmap_cache[key]
         except KeyError:
@@ -159,19 +204,19 @@ class CoverArtThumbnail(ActiveLabel):
                 bgcolor = self.palette().color(QtGui.QPalette.Window)
                 painter = QtGui.QPainter(pixmap)
                 painter.fillRect(QtCore.QRectF(0, 0, stack_width, stack_height), bgcolor)
-                cx = stack_width - w / 2
-                cy = h / 2
+                cx = stack_width - w // 2
+                cy = h // 2
                 if limited:
-                    x, y = (cx - self.shadow.width() / 2, cy - self.shadow.height() / 2)
+                    x, y = (cx - self.shadow.width() // 2, cy - self.shadow.height() // 2)
                     for i in range(3):
                         painter.drawPixmap(x, y, self.shadow)
-                        x -= displacements / 3
-                        y += displacements / 3
+                        x -= displacements // 3
+                        y += displacements // 3
                     cx -= displacements
                     cy += displacements
                 else:
-                    cx = stack_width - w / 2
-                    cy = h / 2
+                    cx = stack_width - w // 2
+                    cy = h // 2
                 for image in reversed(data_to_paint):
                     if isinstance(image, QtGui.QPixmap):
                         thumb = image
@@ -179,7 +224,7 @@ class CoverArtThumbnail(ActiveLabel):
                         thumb = QtGui.QPixmap()
                         thumb.loadFromData(image.data)
                     thumb = self.decorate_cover(thumb)
-                    x, y = (cx - thumb.width() / 2, cy - thumb.height() / 2)
+                    x, y = (cx - thumb.width() // 2, cy - thumb.height() // 2)
                     painter.drawPixmap(x, y, thumb)
                     cx -= displacements
                     cy += displacements
@@ -187,18 +232,19 @@ class CoverArtThumbnail(ActiveLabel):
                     color = QtGui.QColor("darkgoldenrod")
                     border_length = 10
                     for k in range(border_length):
-                        color.setAlpha(255 - k * 255 / border_length)
+                        color.setAlpha(255 - k * 255 // border_length)
                         painter.setPen(color)
                         painter.drawLine(x, y - k - 1, x + 121 + k + 1, y - k - 1)
                         painter.drawLine(x + 121 + k + 2, y - 1 - k, x + 121 + k + 2, y + 121 + 4)
                     for k in range(5):
-                        bgcolor.setAlpha(80 + k * 255 / 7)
+                        bgcolor.setAlpha(80 + k * 255 // 7)
                         painter.setPen(bgcolor)
                         painter.drawLine(x + 121 + 2, y + 121 + 2 + k, x + 121 + border_length + 2, y + 121 + 2 + k)
                 painter.end()
                 pixmap = pixmap.scaled(w, h, QtCore.Qt.KeepAspectRatio, QtCore.Qt.SmoothTransformation)
             self._pixmap_cache[key] = pixmap
 
+        pixmap.setDevicePixelRatio(self.pixel_ratio)
         self.setPixmap(pixmap)
         self.current_pixmap_key = key
 
@@ -218,15 +264,15 @@ class CoverArtThumbnail(ActiveLabel):
             release = metadata.get("musicbrainz_albumid", None)
         if release:
             self.setActive(True)
-            text = _(u"View release on MusicBrainz")
+            text = _("View release on MusicBrainz")
         else:
             self.setActive(False)
             text = ""
         if hasattr(metadata, 'has_common_images'):
             if has_common_images:
-                note = _(u'Common images on all tracks')
+                note = _('Common images on all tracks')
             else:
-                note = _(u'Tracks contain different images')
+                note = _('Tracks contain different images')
             if text:
                 text += '<br />'
             text += '<i>%s</i>' % note
@@ -235,7 +281,7 @@ class CoverArtThumbnail(ActiveLabel):
 
     def open_release_page(self):
         lookup = self.tagger.get_file_lookup()
-        lookup.albumLookup(self.release)
+        lookup.album_lookup(self.release)
 
 
 def set_image_replace(obj, coverartimage):
@@ -246,11 +292,11 @@ def set_image_append(obj, coverartimage):
     obj.metadata.append_image(coverartimage)
 
 
-class CoverArtBox(QtGui.QGroupBox):
+class CoverArtBox(QtWidgets.QGroupBox):
 
     def __init__(self, parent):
-        QtGui.QGroupBox.__init__(self, "")
-        self.layout = QtGui.QVBoxLayout()
+        super().__init__("")
+        self.layout = QtWidgets.QVBoxLayout()
         self.layout.setSpacing(6)
         self.parent = parent
         # Kills off any borders
@@ -258,15 +304,15 @@ class CoverArtBox(QtGui.QGroupBox):
         self.setFlat(True)
         self.item = None
         self.pixmap_cache = LRUCache(40)
-        self.cover_art_label = QtGui.QLabel('')
+        self.cover_art_label = QtWidgets.QLabel('')
         self.cover_art_label.setAlignment(QtCore.Qt.AlignTop | QtCore.Qt.AlignHCenter)
         self.cover_art = CoverArtThumbnail(False, True, self.pixmap_cache, parent)
         self.cover_art.image_dropped.connect(self.fetch_remote_image)
-        spacerItem = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
-        self.orig_cover_art_label = QtGui.QLabel('')
+        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
+        self.orig_cover_art_label = QtWidgets.QLabel('')
         self.orig_cover_art = CoverArtThumbnail(False, False, self.pixmap_cache, parent)
         self.orig_cover_art_label.setAlignment(QtCore.Qt.AlignTop | QtCore.Qt.AlignHCenter)
-        self.show_details_button = QtGui.QPushButton(_(u'Show more details'), self)
+        self.show_details_button = QtWidgets.QPushButton(_('Show more details'), self)
         self.layout.addWidget(self.cover_art_label)
         self.layout.addWidget(self.cover_art)
         self.layout.addWidget(self.orig_cover_art_label)
@@ -303,12 +349,12 @@ class CoverArtBox(QtGui.QGroupBox):
         else:
             self.show_details_button.setVisible(True)
             self.orig_cover_art.setVisible(True)
-            self.cover_art_label.setText(_(u'New Cover Art'))
-            self.orig_cover_art_label.setText(_(u'Original Cover Art'))
+            self.cover_art_label.setText(_('New Cover Art'))
+            self.orig_cover_art_label.setText(_('Original Cover Art'))
 
     def show(self):
         self.update_display(True)
-        super(CoverArtBox, self).show()
+        super().show()
 
     def set_metadata(self, metadata, orig_metadata, item):
         if not metadata or not metadata.images:
@@ -322,31 +368,27 @@ class CoverArtBox(QtGui.QGroupBox):
     def fetch_remote_image(self, url, fallback_data=None):
         if self.item is None:
             return
-        if url.scheme() in ('https', 'http'):
-            path = url.encodedPath()
+
+        if fallback_data:
+            self.load_remote_image(url, None, fallback_data)
+
+        if url.scheme() in ('http', 'https'):
+            path = url.path()
             if url.hasQuery():
-                path += '?' + url.encodedQuery()
+                query = QtCore.QUrlQuery(url.query())
+                queryargs = dict(query.queryItems())
+            else:
+                queryargs = {}
             if url.scheme() == 'https':
                 port = 443
             else:
                 port = 80
-            self.tagger.xmlws.get(str(url.encodedHost()), url.port(port), str(path),
-                                  partial(self.on_remote_image_fetched, url, fallback_data=fallback_data),
-                                  xml=False,
-                                  priority=True, important=True)
+            self.tagger.webservice.get(url.host(), url.port(port), path,
+                                       partial(self.on_remote_image_fetched, url, fallback_data=fallback_data),
+                                       parse_response_type=None, queryargs=queryargs,
+                                       priority=True, important=True)
         elif url.scheme() == 'file':
-            log.debug("Dropped the URL: %r", url.toString(QtCore.QUrl.RemoveUserInfo))
-            if sys.platform == 'darwin' and unicode(url.path()).startswith('/.file/id='):
-                # Workaround for https://bugreports.qt.io/browse/QTBUG-40449
-                # OSX Urls follow the NSURL scheme and need to be converted
-                if NSURL_IMPORTED:
-                    path = os.path.normpath(os.path.realpath(unicode(NSURL.URLWithString_(str(url.toString())).filePathURL().path()).rstrip("\0")))
-                    log.debug('OSX NSURL path detected. Dropped File is: %r', path)
-                else:
-                    log.error("Unable to get appropriate file path for %r", url.toString(QtCore.QUrl.RemoveUserInfo))
-            else:
-                # Dropping a file from iTunes gives a path with a NULL terminator
-                path = os.path.normpath(os.path.realpath(unicode(url.toLocalFile()).rstrip("\0")))
+            path = os.path.normpath(os.path.realpath(url.toLocalFile().rstrip("\0")))
             if path and os.path.exists(path):
                 mime = 'image/png' if path.lower().endswith('.png') else 'image/jpeg'
                 with open(path, 'rb') as f:
@@ -354,12 +396,14 @@ class CoverArtBox(QtGui.QGroupBox):
                 self.load_remote_image(url, mime, data)
 
     def on_remote_image_fetched(self, url, data, reply, error, fallback_data=None):
+        data = bytes(data)
         mime = reply.header(QtNetwork.QNetworkRequest.ContentTypeHeader)
+        url_query = QtCore.QUrlQuery(url.query())
         if mime in ('image/jpeg', 'image/png'):
             self.load_remote_image(url, mime, data)
-        elif url.hasQueryItem("imgurl"):
+        elif url_query.hasQueryItem("imgurl"):
             # This may be a google images result, try to get the URL which is encoded in the query
-            url = QtCore.QUrl(url.queryItemValue("imgurl"))
+            url = QtCore.QUrl(url_query.queryItemValue("imgurl", QtCore.QUrl.FullyDecoded))
             self.fetch_remote_image(url)
         else:
             log.warning("Can't load remote image with MIME-Type %s", mime)
@@ -370,25 +414,28 @@ class CoverArtBox(QtGui.QGroupBox):
                 except imageinfo.IdentificationError as e:
                     log.error("Unable to identify dropped data format: %s" % e)
                 else:
-                    log.debug("Trying the dropped %s data", mime)
                     self.load_remote_image(url, mime, fallback_data)
+                    log.debug("Trying the dropped %s data", mime)
 
     def load_remote_image(self, url, mime, data):
         try:
             coverartimage = CoverArtImage(
                 url=url.toString(),
-                types=[u'front'],
+                types=['front'],
                 data=data
             )
         except CoverArtImageError as e:
-            log.warning("Can't load image: %s" % unicode(e))
+            log.warning("Can't load image: %s" % e)
             return
 
         if config.setting["load_image_behavior"] == 'replace':
             set_image = set_image_replace
+            debug_info = "Replacing with dropped %r in %r"
         else:
             set_image = set_image_append
+            debug_info = "Appending dropped %r to %r"
 
+        update = True
         if isinstance(self.item, Album):
             album = self.item
             album.enable_update_metadata_images(False)
@@ -420,36 +467,43 @@ class CoverArtBox(QtGui.QGroupBox):
             set_image(file, coverartimage)
             file.metadata_images_changed.emit()
             file.update()
-        self.cover_art.set_metadata(self.item.metadata)
-        self.show()
+        else:
+            debug_info = "Dropping %r to %r is not handled"
+            update = False
+
+        log.debug(debug_info, coverartimage, self.item)
+
+        if update:
+            self.cover_art.set_metadata(self.item.metadata)
+            self.show()
 
     def set_load_image_behavior(self, behavior):
         config.setting["load_image_behavior"] = behavior
 
     def contextMenuEvent(self, event):
-        menu = QtGui.QMenu(self)
+        menu = QtWidgets.QMenu(self)
         if self.show_details_button.isVisible():
-            name = _(u'Show more details...')
-            show_more_details_action = QtGui.QAction(name, self.parent)
+            name = _('Show more details...')
+            show_more_details_action = QtWidgets.QAction(name, self.parent)
             show_more_details_action.triggered.connect(self.show_cover_art_info)
             menu.addAction(show_more_details_action)
 
         if self.orig_cover_art.isVisible():
-            name = _(u'Keep original cover art')
-            use_orig_value_action = QtGui.QAction(name, self.parent)
+            name = _('Keep original cover art')
+            use_orig_value_action = QtWidgets.QAction(name, self.parent)
             use_orig_value_action.triggered.connect(self.item.keep_original_images)
             menu.addAction(use_orig_value_action)
 
         if not menu.isEmpty():
             menu.addSeparator()
 
-        load_image_behavior_group = QtGui.QActionGroup(self.parent, exclusive=True)
-        action = load_image_behavior_group.addAction(QtGui.QAction(_(u'Replace front cover art on drop'), self.parent, checkable=True))
+        load_image_behavior_group = QtWidgets.QActionGroup(self.parent, exclusive=True)
+        action = load_image_behavior_group.addAction(QtWidgets.QAction(_('Replace front cover art on drop'), self.parent, checkable=True))
         action.triggered.connect(partial(self.set_load_image_behavior, behavior='replace'))
         if config.setting["load_image_behavior"] == 'replace':
             action.setChecked(True)
         menu.addAction(action)
-        action = load_image_behavior_group.addAction(QtGui.QAction(_(u'Append front cover art on drop'), self.parent, checkable=True))
+        action = load_image_behavior_group.addAction(QtWidgets.QAction(_('Append front cover art on drop'), self.parent, checkable=True))
         action.triggered.connect(partial(self.set_load_image_behavior, behavior='append'))
         if config.setting["load_image_behavior"] == 'append':
             action.setChecked(True)
