@@ -18,21 +18,32 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-from PyQt4 import QtCore, QtGui
 from collections import defaultdict
 from functools import partial
+
+from PyQt5 import (
+    QtCore,
+    QtGui,
+    QtWidgets,
+)
+
 from picard import config
 from picard.album import Album
-from picard.cluster import Cluster
-from picard.track import Track
-from picard.file import File
-from picard.util import format_time, throttle, thread, uniqify
-from picard.util.tags import display_tag_name
-from picard.ui.edittagdialog import EditTagDialog
-from picard.metadata import MULTI_VALUED_JOINER
-from picard.browser.filelookup import FileLookup
 from picard.browser.browser import BrowserIntegration
+from picard.browser.filelookup import FileLookup
+from picard.cluster import Cluster
+from picard.file import File
+from picard.metadata import MULTI_VALUED_JOINER
+from picard.track import Track
+from picard.util import (
+    format_time,
+    restore_method,
+    thread,
+    throttle,
+)
+from picard.util.tags import display_tag_name
 
+from picard.ui.edittagdialog import EditTagDialog
 
 COMMON_TAGS = [
     "title",
@@ -65,7 +76,7 @@ class TagCounter(dict):
         self.different = set()
 
     def __getitem__(self, tag):
-        return dict.get(self, tag, [""])
+        return super().get(tag, [""])
 
     def add(self, tag, values):
         if tag not in self.different:
@@ -81,7 +92,7 @@ class TagCounter(dict):
         missing = self.parent.objects - count
 
         if tag in self.different:
-            return (ungettext("(different across %d item)", "(different across %d items)", count) % count, True)
+            return (ngettext("(different across %d item)", "(different across %d items)", count) % count, True)
         else:
             if tag == "~length":
                 msg = format_time(self.get(tag, 0))
@@ -89,25 +100,26 @@ class TagCounter(dict):
                 msg = MULTI_VALUED_JOINER.join(self[tag])
 
             if count > 0 and missing > 0:
-                return (msg + " " + (ungettext("(missing from %d item)", "(missing from %d items)", missing) % missing), True)
+                return (msg + " " + (ngettext("(missing from %d item)", "(missing from %d items)", missing) % missing), True)
             else:
                 return (msg, False)
 
 
 class TagDiff(object):
 
-    __slots__ = ("tag_names", "new", "orig", "status", "objects")
+    __slots__ = ("tag_names", "new", "orig", "status", "objects", "max_length_delta_ms")
 
-    def __init__(self):
+    def __init__(self, max_length_diff=2):
         self.tag_names = []
         self.new = TagCounter(self)
         self.orig = TagCounter(self)
         self.status = defaultdict(lambda: 0)
         self.objects = 0
+        self.max_length_delta_ms = max_length_diff * 1000
 
     def __tag_ne(self, tag, orig, new):
         if tag == "~length":
-            return abs(float(orig) - float(new)) > 2000
+            return abs(float(orig) - float(new)) > self.max_length_delta_ms
         else:
             return orig != new
 
@@ -142,7 +154,33 @@ class TagDiff(object):
         return TagStatus.NoChange
 
 
-class MetadataBox(QtGui.QTableWidget):
+class PreservedTags:
+
+    opt_name = 'preserved_tags'
+
+    def __init__(self):
+        self._tags = self._from_config()
+
+    def _to_config(self):
+        config.setting[self.opt_name] = ", ".join(sorted(self._tags))
+
+    def _from_config(self):
+        tags = config.setting[self.opt_name].split(',')
+        return set(filter(bool, map(str.strip, tags)))
+
+    def add(self, name):
+        self._tags.add(name)
+        self._to_config()
+
+    def discard(self, name):
+        self._tags.discard(name)
+        self._to_config()
+
+    def __contains__(self, key):
+        return key in self._tags
+
+
+class MetadataBox(QtWidgets.QTableWidget):
 
     options = (
         config.Option("persist", "metadatabox_header_state", QtCore.QByteArray()),
@@ -150,19 +188,19 @@ class MetadataBox(QtGui.QTableWidget):
     )
 
     def __init__(self, parent):
-        QtGui.QTableWidget.__init__(self, parent)
+        super().__init__(parent)
         self.parent = parent
         self.setAccessibleName(_("metadata view"))
         self.setAccessibleDescription(_("Displays original and new tags for the selected files"))
         self.setColumnCount(3)
         self.setHorizontalHeaderLabels((_("Tag"), _("Original Value"), _("New Value")))
         self.horizontalHeader().setStretchLastSection(True)
-        self.horizontalHeader().setResizeMode(QtGui.QHeaderView.Stretch)
-        self.horizontalHeader().setClickable(False)
+        self.horizontalHeader().setSectionResizeMode(QtWidgets.QHeaderView.Stretch)
+        self.horizontalHeader().setSectionsClickable(False)
         self.verticalHeader().setDefaultSectionSize(21)
         self.verticalHeader().setVisible(False)
-        self.setHorizontalScrollMode(QtGui.QAbstractItemView.ScrollPerPixel)
-        self.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
+        self.setHorizontalScrollMode(QtWidgets.QAbstractItemView.ScrollPerPixel)
+        self.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
         self.setTabKeyNavigation(False)
         self.setStyleSheet("QTableWidget {border: none;}")
         self.setAttribute(QtCore.Qt.WA_MacShowFocusRect, 1)
@@ -179,20 +217,21 @@ class MetadataBox(QtGui.QTableWidget):
         self.selection_dirty = False
         self.editing = None  # the QTableWidgetItem being edited
         self.clipboard = [""]
-        self.add_tag_action = QtGui.QAction(_(u"Add New Tag..."), parent)
+        self.add_tag_action = QtWidgets.QAction(_("Add New Tag..."), parent)
         self.add_tag_action.triggered.connect(partial(self.edit_tag, ""))
-        self.changes_first_action = QtGui.QAction(_(u"Show Changes First"), parent)
+        self.changes_first_action = QtWidgets.QAction(_("Show Changes First"), parent)
         self.changes_first_action.setCheckable(True)
         self.changes_first_action.setChecked(config.persist["show_changes_first"])
         self.changes_first_action.toggled.connect(self.toggle_changes_first)
         self.browser_integration = BrowserIntegration()
         # TR: Keyboard shortcut for "Add New Tag..."
-        self.add_tag_shortcut = QtGui.QShortcut(QtGui.QKeySequence(_("Alt+Shift+A")), self, partial(self.edit_tag, ""))
+        self.add_tag_shortcut = QtWidgets.QShortcut(QtGui.QKeySequence(_("Alt+Shift+A")), self, partial(self.edit_tag, ""))
         self.add_tag_action.setShortcut(self.add_tag_shortcut.key())
         # TR: Keyboard shortcut for "Edit..." (tag)
-        self.edit_tag_shortcut = QtGui.QShortcut(QtGui.QKeySequence(_("Alt+Shift+E")), self, partial(self.edit_selected_tag))
+        self.edit_tag_shortcut = QtWidgets.QShortcut(QtGui.QKeySequence(_("Alt+Shift+E")), self, partial(self.edit_selected_tag))
         # TR: Keyboard shortcut for "Remove" (tag)
-        self.remove_tag_shortcut = QtGui.QShortcut(QtGui.QKeySequence(_("Alt+Shift+R")), self, self.remove_selected_tags)
+        self.remove_tag_shortcut = QtWidgets.QShortcut(QtGui.QKeySequence(_("Alt+Shift+R")), self, self.remove_selected_tags)
+        self.preserved_tags = PreservedTags()
 
     def get_file_lookup(self):
         """Return a FileLookup object."""
@@ -203,14 +242,14 @@ class MetadataBox(QtGui.QTableWidget):
     def lookup_tags(self):
         lookup = self.get_file_lookup()
         LOOKUP_TAGS = {
-            "musicbrainz_recordingid": lookup.recordingLookup,
-            "musicbrainz_trackid": lookup.trackLookup,
-            "musicbrainz_albumid": lookup.albumLookup,
-            "musicbrainz_workid": lookup.workLookup,
-            "musicbrainz_artistid": lookup.artistLookup,
-            "musicbrainz_albumartistid": lookup.artistLookup,
-            "musicbrainz_releasegroupid": lookup.releaseGroupLookup,
-            "acoustid_id": lookup.acoustLookup
+            "musicbrainz_recordingid": lookup.recording_lookup,
+            "musicbrainz_trackid": lookup.track_lookup,
+            "musicbrainz_albumid": lookup.album_lookup,
+            "musicbrainz_workid": lookup.work_lookup,
+            "musicbrainz_artistid": lookup.artist_lookup,
+            "musicbrainz_albumartistid": lookup.artist_lookup,
+            "musicbrainz_releasegroupid": lookup.release_group_lookup,
+            "acoustid_id": lookup.acoust_lookup
         }
         return LOOKUP_TAGS
 
@@ -225,9 +264,9 @@ class MetadataBox(QtGui.QTableWidget):
             return False
         item = self.itemFromIndex(index)
         if item.flags() & QtCore.Qt.ItemIsEditable and \
-           trigger in (QtGui.QAbstractItemView.DoubleClicked,
-                       QtGui.QAbstractItemView.EditKeyPressed,
-                       QtGui.QAbstractItemView.AnyKeyPressed):
+           trigger in (QtWidgets.QAbstractItemView.DoubleClicked,
+                       QtWidgets.QAbstractItemView.EditKeyPressed,
+                       QtWidgets.QAbstractItemView.AnyKeyPressed):
             tag = self.tag_diff.tag_names[item.row()]
             values = self.tag_diff.new[tag]
             if len(values) > 1:
@@ -236,7 +275,7 @@ class MetadataBox(QtGui.QTableWidget):
             else:
                 self.editing = item
                 item.setText(values[0])
-                return QtGui.QTableWidget.edit(self, index, trigger, event)
+                return super().edit(index, trigger, event)
         return False
 
     def event(self, e):
@@ -251,13 +290,13 @@ class MetadataBox(QtGui.QTableWidget):
                     self.clipboard = list(self.tag_diff.new[tag])
             elif e.key() == QtCore.Qt.Key_V and column == 2 and tag != "~length":
                 self.set_tag_values(tag, list(self.clipboard))
-        return QtGui.QTableWidget.event(self, e)
+        return super().event(e)
 
     def closeEditor(self, editor, hint):
-        QtGui.QTableWidget.closeEditor(self, editor, hint)
+        super().closeEditor(editor, hint)
         tag = self.tag_diff.tag_names[self.editing.row()]
         old = self.tag_diff.new[tag]
-        new = [unicode(editor.text())]
+        new = [editor.text()]
         if old == new:
             self.editing.setText(old[0])
         else:
@@ -266,61 +305,59 @@ class MetadataBox(QtGui.QTableWidget):
         self.update()
 
     def contextMenuEvent(self, event):
-        menu = QtGui.QMenu(self)
+        menu = QtWidgets.QMenu(self)
         if self.objects:
-            tags = self.selected_tags(discard=('~length'))
+            tags = self.selected_tags(discard=('~length',))
             if len(tags) == 1:
                 selected_tag = tags[0]
-                edit_tag_action = QtGui.QAction(_(u"Edit..."), self.parent)
+                edit_tag_action = QtWidgets.QAction(_("Edit..."), self.parent)
                 edit_tag_action.triggered.connect(partial(self.edit_tag, selected_tag))
                 edit_tag_action.setShortcut(self.edit_tag_shortcut.key())
                 menu.addAction(edit_tag_action)
-                preserved_tags = [tag for tag in
-                                  map(lambda x: x.strip(), config.setting['preserved_tags'].split(','))
-                                  if tag != ""]
-                if selected_tag not in preserved_tags:
-                    add_to_preserved_tags_action = QtGui.QAction(_(u"Add to 'Preserve Tags' List"), self.parent)
-                    add_to_preserved_tags_action.triggered.connect(partial(self.add_to_preserved_tags, selected_tag, preserved_tags))
+                if selected_tag not in self.preserved_tags:
+                    add_to_preserved_tags_action = QtWidgets.QAction(_("Add to 'Preserve Tags' List"), self.parent)
+                    add_to_preserved_tags_action.triggered.connect(partial(self.preserved_tags.add, selected_tag))
                     menu.addAction(add_to_preserved_tags_action)
                 else:
-                    remove_from_preserved_tags_action = QtGui.QAction(_(u"Remove from 'Preserve Tags' List"), self.parent)
-                    remove_from_preserved_tags_action.triggered.connect(partial(self.remove_from_preserved_tags, selected_tag, preserved_tags))
+                    remove_from_preserved_tags_action = QtWidgets.QAction(_("Remove from 'Preserve Tags' List"), self.parent)
+                    remove_from_preserved_tags_action.triggered.connect(partial(self.preserved_tags.discard, selected_tag))
                     menu.addAction(remove_from_preserved_tags_action)
             removals = []
             useorigs = []
             item = self.currentItem()
-            column = item.column()
-            for tag in tags:
-                if tag in self.lookup_tags().keys():
-                    if (column == 1 or column == 2) and len(tags) == 1 and item.text():
-                        if column == 1:
-                            values = self.tag_diff.orig[tag]
-                        else:
-                            values = self.tag_diff.new[tag]
-                        lookup_action = QtGui.QAction(_(u"Lookup in &Browser"), self.parent)
-                        lookup_action.triggered.connect(partial(self.open_link, values, tag))
-                        menu.addAction(lookup_action)
-                if self.tag_is_removable(tag):
-                    removals.append(partial(self.remove_tag, tag))
-                status = self.tag_diff.status[tag] & TagStatus.Changed
-                if status == TagStatus.Changed or status == TagStatus.Removed:
-                    for file in self.files:
-                        objects = [file]
-                        if file.parent in self.tracks and len(self.files & set(file.parent.linked_files)) == 1:
-                            objects.append(file.parent)
-                        orig_values = list(file.orig_metadata.getall(tag)) or [""]
-                        useorigs.append(partial(self.set_tag_values, tag, orig_values, objects))
-            if removals:
-                remove_tag_action = QtGui.QAction(_(u"Remove"), self.parent)
-                remove_tag_action.triggered.connect(lambda: [f() for f in removals])
-                remove_tag_action.setShortcut(self.remove_tag_shortcut.key())
-                menu.addAction(remove_tag_action)
-            if useorigs:
-                name = ungettext("Use Original Value", "Use Original Values", len(useorigs))
-                use_orig_value_action = QtGui.QAction(name, self.parent)
-                use_orig_value_action.triggered.connect(lambda: [f() for f in useorigs])
-                menu.addAction(use_orig_value_action)
-                menu.addSeparator()
+            if item:
+                column = item.column()
+                for tag in tags:
+                    if tag in self.lookup_tags().keys():
+                        if (column == 1 or column == 2) and len(tags) == 1 and item.text():
+                            if column == 1:
+                                values = self.tag_diff.orig[tag]
+                            else:
+                                values = self.tag_diff.new[tag]
+                            lookup_action = QtWidgets.QAction(_("Lookup in &Browser"), self.parent)
+                            lookup_action.triggered.connect(partial(self.open_link, values, tag))
+                            menu.addAction(lookup_action)
+                    if self.tag_is_removable(tag):
+                        removals.append(partial(self.remove_tag, tag))
+                    status = self.tag_diff.status[tag] & TagStatus.Changed
+                    if status == TagStatus.Changed or status == TagStatus.Removed:
+                        for file in self.files:
+                            objects = [file]
+                            if file.parent in self.tracks and len(self.files & set(file.parent.linked_files)) == 1:
+                                objects.append(file.parent)
+                            orig_values = list(file.orig_metadata.getall(tag)) or [""]
+                            useorigs.append(partial(self.set_tag_values, tag, orig_values, objects))
+                if removals:
+                    remove_tag_action = QtWidgets.QAction(_("Remove"), self.parent)
+                    remove_tag_action.triggered.connect(lambda: [f() for f in removals])
+                    remove_tag_action.setShortcut(self.remove_tag_shortcut.key())
+                    menu.addAction(remove_tag_action)
+                if useorigs:
+                    name = ngettext("Use Original Value", "Use Original Values", len(useorigs))
+                    use_orig_value_action = QtWidgets.QAction(name, self.parent)
+                    use_orig_value_action.triggered.connect(lambda: [f() for f in useorigs])
+                    menu.addAction(use_orig_value_action)
+                    menu.addSeparator()
             if len(tags) == 1 or removals or useorigs:
                 menu.addSeparator()
             menu.addAction(self.add_tag_action)
@@ -329,23 +366,11 @@ class MetadataBox(QtGui.QTableWidget):
         menu.exec_(event.globalPos())
         event.accept()
 
-    @staticmethod
-    def _save_preserved_tags(preserved_tags):
-        config.setting['preserved_tags'] = ", ".join(uniqify(preserved_tags))
-
-    def add_to_preserved_tags(self, name, preserved_tags):
-        preserved_tags.append(name)
-        self._save_preserved_tags(preserved_tags)
-
-    def remove_from_preserved_tags(self, name, preserved_tags):
-        preserved_tags = filter(lambda x: x != name, preserved_tags)
-        self._save_preserved_tags(preserved_tags)
-
     def edit_tag(self, tag):
         EditTagDialog(self.parent, tag).exec_()
 
     def edit_selected_tag(self):
-        tags = self.selected_tags(discard=('~length'))
+        tags = self.selected_tags(discard=('~length',))
         if len(tags) == 1:
             self.edit_tag(tags[0])
 
@@ -357,7 +382,12 @@ class MetadataBox(QtGui.QTableWidget):
         if objects is None:
             objects = self.objects
         self.parent.ignore_selection_changes = True
-        if values != [""] or self.tag_is_removable(tag):
+        if values == [""]:
+            values = []
+        if not values and self.tag_is_removable(tag):
+            for obj in objects:
+                obj.metadata.delete(tag)
+        elif values:
             for obj in objects:
                 obj.metadata[tag] = values
                 obj.update()
@@ -365,10 +395,10 @@ class MetadataBox(QtGui.QTableWidget):
         self.parent.ignore_selection_changes = False
 
     def remove_tag(self, tag):
-        self.set_tag_values(tag, [""])
+        self.set_tag_values(tag, [])
 
     def remove_selected_tags(self):
-        for tag in self.selected_tags(discard=('~length')):
+        for tag in self.selected_tags(discard=('~length',)):
             if self.tag_is_removable(tag):
                 self.remove_tag(tag)
 
@@ -427,7 +457,7 @@ class MetadataBox(QtGui.QTableWidget):
         if not (files or tracks):
             return None
 
-        tag_diff = TagDiff()
+        tag_diff = TagDiff(max_length_diff=config.setting["ignore_track_duration_difference_under"])
         orig_tags = tag_diff.orig
         new_tags = tag_diff.new
         # existing_tags are orig_tags that would not be overwritten by
@@ -440,7 +470,7 @@ class MetadataBox(QtGui.QTableWidget):
         for file in files:
             new_metadata = file.new_metadata
             orig_metadata = file.orig_metadata
-            tags = set(new_metadata.keys() + orig_metadata.keys())
+            tags = set(list(new_metadata.keys()) + list(orig_metadata.keys()))
 
             for name in filter(lambda x: not x.startswith("~") and file.supports_tag(x), tags):
                 new_values = new_metadata.getall(name)
@@ -458,7 +488,7 @@ class MetadataBox(QtGui.QTableWidget):
 
         for track in tracks:
             if track.num_linked_files == 0:
-                for name, values in dict.iteritems(track.metadata):
+                for name, values in track.metadata.rawitems():
                     if not name.startswith("~"):
                         tag_diff.add(name, values, values, True)
 
@@ -467,10 +497,10 @@ class MetadataBox(QtGui.QTableWidget):
 
                 tag_diff.objects += 1
 
-        all_tags = set(orig_tags.keys() + new_tags.keys())
+        all_tags = set(list(orig_tags.keys()) + list(new_tags.keys()))
         tag_names = COMMON_TAGS + \
-                    sorted(all_tags.difference(COMMON_TAGS),
-                           key=lambda x: display_tag_name(x).lower())
+            sorted(all_tags.difference(COMMON_TAGS),
+                   key=lambda x: display_tag_name(x).lower())
 
         if config.persist["show_changes_first"]:
             tags_by_status = {}
@@ -507,27 +537,29 @@ class MetadataBox(QtGui.QTableWidget):
         new_flags = orig_flags | QtCore.Qt.ItemIsEditable
 
         for i, name in enumerate(result.tag_names):
-            length = name == "~length"
             tag_item = self.item(i, 0)
             orig_item = self.item(i, 1)
             new_item = self.item(i, 2)
             if not tag_item:
-                tag_item = QtGui.QTableWidgetItem()
+                tag_item = QtWidgets.QTableWidgetItem()
                 tag_item.setFlags(orig_flags)
                 font = tag_item.font()
                 font.setBold(True)
                 tag_item.setFont(font)
                 self.setItem(i, 0, tag_item)
             if not orig_item:
-                orig_item = QtGui.QTableWidgetItem()
+                orig_item = QtWidgets.QTableWidgetItem()
                 orig_item.setFlags(orig_flags)
                 self.setItem(i, 1, orig_item)
             if not new_item:
-                new_item = QtGui.QTableWidgetItem()
+                new_item = QtWidgets.QTableWidgetItem()
                 self.setItem(i, 2, new_item)
             tag_item.setText(display_tag_name(name))
             self.set_item_value(orig_item, self.tag_diff.orig, name)
-            new_item.setFlags(orig_flags if length else new_flags)
+            if name == "~length":
+                new_item.setFlags(orig_flags)
+            else:
+                new_item.setFlags(new_flags)
             self.set_item_value(new_item, self.tag_diff.new, name)
 
             font = new_item.font()
@@ -550,11 +582,12 @@ class MetadataBox(QtGui.QTableWidget):
         font.setItalic(italic)
         item.setFont(font)
 
+    @restore_method
     def restore_state(self):
         state = config.persist["metadatabox_header_state"]
         header = self.horizontalHeader()
         header.restoreState(state)
-        header.setResizeMode(QtGui.QHeaderView.Interactive)
+        header.setSectionResizeMode(QtWidgets.QHeaderView.Interactive)
 
     def save_state(self):
         header = self.horizontalHeader()
