@@ -17,17 +17,29 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-import os.path
 from functools import partial
-from PyQt4 import QtCore, QtGui
-from picard import config, log
+import locale
+import os.path
+
+from PyQt5 import (
+    QtCore,
+    QtWidgets,
+)
+from PyQt5.QtCore import QStandardPaths
+
+from picard import config
+from picard.const import UI_LANGUAGES
 from picard.util import icontheme
-from picard.ui.options import OptionsPage, register_options_page
+
+from picard.ui.moveable_list_view import MoveableListView
+from picard.ui.options import (
+    OptionsPage,
+    register_options_page,
+)
 from picard.ui.ui_options_interface import Ui_InterfaceOptionsPage
 from picard.ui.util import enabledSlot
-from picard.const import UI_LANGUAGES
-import operator
-import locale
+
+_default_starting_dir = QStandardPaths.writableLocation(QStandardPaths.HomeLocation)
 
 
 class InterfaceOptionsPage(OptionsPage):
@@ -37,54 +49,54 @@ class InterfaceOptionsPage(OptionsPage):
     PARENT = None
     SORT_ORDER = 80
     ACTIVE = True
-    SEPARATOR = u'—'*5
+    SEPARATOR = '—'*5
     TOOLBAR_BUTTONS = {
         'add_directory_action': {
-            'label': N_(u'Add Folder'),
+            'label': N_('Add Folder'),
             'icon': 'folder'
         },
-       'add_files_action': {
-            'label': N_(u'Add Files'),
+        'add_files_action': {
+            'label': N_('Add Files'),
             'icon': 'document-open'
         },
-       'cluster_action': {
-            'label': N_(u'Cluster'),
+        'cluster_action': {
+            'label': N_('Cluster'),
             'icon': 'picard-cluster'
         },
-       'autotag_action': {
-            'label': N_(u'Lookup'),
+        'autotag_action': {
+            'label': N_('Lookup'),
             'icon': 'picard-auto-tag'
         },
-       'analyze_action': {
-            'label': N_(u'Scan'),
+        'analyze_action': {
+            'label': N_('Scan'),
             'icon': 'picard-analyze'
         },
-       'browser_lookup_action': {
-            'label': N_(u'Lookup in Browser'),
+        'browser_lookup_action': {
+            'label': N_('Lookup in Browser'),
             'icon': 'lookup-musicbrainz'
         },
-       'save_action': {
-            'label': N_(u'Save'),
+        'save_action': {
+            'label': N_('Save'),
             'icon': 'document-save'
         },
-       'view_info_action': {
-            'label': N_(u'Info'),
+        'view_info_action': {
+            'label': N_('Info'),
             'icon': 'picard-edit-tags'
         },
-       'remove_action': {
-            'label': N_(u'Remove'),
+        'remove_action': {
+            'label': N_('Remove'),
             'icon': 'list-remove'
         },
-       'submit_acoustid_action': {
-            'label': N_(u'Submit AcoustIDs'),
+        'submit_acoustid_action': {
+            'label': N_('Submit AcoustIDs'),
             'icon': 'acoustid-fingerprinter'
         },
-       'play_file_action': {
-            'label': N_(u'Open in Player'),
+        'play_file_action': {
+            'label': N_('Open in Player'),
             'icon': 'play-music'
         },
-       'cd_lookup_action': {
-            'label': N_(u'Lookup CD...'),
+        'cd_lookup_action': {
+            'label': N_('Lookup CD...'),
             'icon': 'media-optical'
         },
     }
@@ -95,9 +107,9 @@ class InterfaceOptionsPage(OptionsPage):
         config.BoolOption("setting", "builtin_search", False),
         config.BoolOption("setting", "use_adv_search_syntax", False),
         config.BoolOption("setting", "quit_confirmation", True),
-        config.TextOption("setting", "ui_language", u""),
+        config.TextOption("setting", "ui_language", ""),
         config.BoolOption("setting", "starting_directory", False),
-        config.TextOption("setting", "starting_directory_path", ""),
+        config.TextOption("setting", "starting_directory_path", _default_starting_dir),
         config.TextOption("setting", "load_image_behavior", "append"),
         config.ListOption("setting", "toolbar_layout", [
             'add_directory_action',
@@ -120,15 +132,16 @@ class InterfaceOptionsPage(OptionsPage):
     ]
 
     def __init__(self, parent=None):
-        super(InterfaceOptionsPage, self).__init__(parent)
+        super().__init__(parent)
         self.ui = Ui_InterfaceOptionsPage()
         self.ui.setupUi(self)
         self.ui.ui_language.addItem(_('System default'), '')
         language_list = [(l[0], l[1], _(l[2])) for l in UI_LANGUAGES]
-        for lang_code, native, translation in sorted(language_list, key=operator.itemgetter(2),
-                                                     cmp=locale.strcoll):
+
+        def fcmp(x): return locale.strxfrm(x[2])
+        for lang_code, native, translation in sorted(language_list, key=fcmp):
             if native and native != translation:
-                name = u'%s (%s)' % (translation, native)
+                name = '%s (%s)' % (translation, native)
             else:
                 name = translation
             self.ui.ui_language.addItem(name, lang_code)
@@ -148,11 +161,9 @@ class InterfaceOptionsPage(OptionsPage):
         self.ui.add_button.clicked.connect(self.add_to_toolbar)
         self.ui.insert_separator_button.clicked.connect(self.insert_separator)
         self.ui.remove_button.clicked.connect(self.remove_action)
-        self.ui.up_button.clicked.connect(partial(self.move_item, 1))
-        self.ui.down_button.clicked.connect(partial(self.move_item, -1))
-        self.ui.toolbar_layout_list.currentRowChanged.connect(self.update_buttons)
-        self.ui.toolbar_layout_list.setDragDropMode(QtGui.QAbstractItemView.DragDrop)
-        self.ui.toolbar_layout_list.setDefaultDropAction(QtCore.Qt.MoveAction)
+        self.move_view = MoveableListView(self.ui.toolbar_layout_list, self.ui.up_button,
+                                          self.ui.down_button, self.update_action_buttons)
+        self.update_buttons = self.move_view.update_buttons
 
     def load(self):
         self.ui.toolbar_show_labels.setChecked(config.setting["toolbar_show_labels"])
@@ -178,26 +189,26 @@ class InterfaceOptionsPage(OptionsPage):
         new_language = self.ui.ui_language.itemData(self.ui.ui_language.currentIndex())
         if new_language != config.setting["ui_language"]:
             config.setting["ui_language"] = self.ui.ui_language.itemData(self.ui.ui_language.currentIndex())
-            dialog = QtGui.QMessageBox(
-                QtGui.QMessageBox.Information,
+            dialog = QtWidgets.QMessageBox(
+                QtWidgets.QMessageBox.Information,
                 _('Language changed'),
                 _('You have changed the interface language. You have to restart Picard in order for the change to take effect.'),
-                QtGui.QMessageBox.Ok,
+                QtWidgets.QMessageBox.Ok,
                 self)
             dialog.exec_()
         config.setting["starting_directory"] = self.ui.starting_directory.isChecked()
-        config.setting["starting_directory_path"] = os.path.normpath(unicode(self.ui.starting_directory_path.text()))
+        config.setting["starting_directory_path"] = os.path.normpath(self.ui.starting_directory_path.text())
         self.update_layout_config()
 
     def restore_defaults(self):
-        super(InterfaceOptionsPage, self).restore_defaults()
+        super().restore_defaults()
         self.update_buttons()
 
     def starting_directory_browse(self):
         item = self.ui.starting_directory_path
-        path = QtGui.QFileDialog.getExistingDirectory(self, "", item.text())
+        path = QtWidgets.QFileDialog.getExistingDirectory(self, "", item.text())
         if path:
-            path = os.path.normpath(unicode(path))
+            path = os.path.normpath(path)
             item.setText(path)
 
     def _get_icon_from_name(self, name):
@@ -205,9 +216,11 @@ class InterfaceOptionsPage(OptionsPage):
 
     def _insert_item(self, action, index=None):
         list_item = ToolbarListItem(action)
-        list_item.setToolTip(_(u'Drag and Drop to re-order'))
+        list_item.setToolTip(_('Drag and Drop to re-order'))
         if action in self.TOOLBAR_BUTTONS:
-            list_item.setText(_(self.TOOLBAR_BUTTONS[action]['label']))
+            # TODO: Remove temporary workaround once https://github.com/python-babel/babel/issues/415 has been resolved.
+            babel_415_workaround = self.TOOLBAR_BUTTONS[action]['label']
+            list_item.setText(_(babel_415_workaround))
             list_item.setIcon(icontheme.lookup(self._get_icon_from_name(action), icontheme.ICON_SIZE_MENU))
         else:
             list_item.setText(self.SEPARATOR)
@@ -223,8 +236,7 @@ class InterfaceOptionsPage(OptionsPage):
 
     def _added_actions(self):
         actions = self._all_list_items()
-        actions = filter(lambda x: x != 'separator', actions)
-        return set(actions)
+        return set(action for action in actions if action != 'separator')
 
     def populate_action_list(self):
         self.ui.toolbar_layout_list.clear()
@@ -232,11 +244,8 @@ class InterfaceOptionsPage(OptionsPage):
             if name in self.ACTION_NAMES or name == 'separator':
                 self._insert_item(name)
 
-    def update_buttons(self):
+    def update_action_buttons(self):
         self.ui.add_button.setEnabled(self._added_actions() != self.ACTION_NAMES)
-        current_row = self.ui.toolbar_layout_list.currentRow()
-        self.ui.up_button.setEnabled(current_row > 0)
-        self.ui.down_button.setEnabled(current_row < self.ui.toolbar_layout_list.count() - 1)
 
     def add_to_toolbar(self):
         display_list = set.difference(self.ACTION_NAMES, self._added_actions())
@@ -250,16 +259,6 @@ class InterfaceOptionsPage(OptionsPage):
         insert_index = self.ui.toolbar_layout_list.currentRow() + 1
         self._insert_item('separator', insert_index)
 
-    def move_item(self, offset):
-        current_index = self.ui.toolbar_layout_list.currentRow()
-        offset_index = current_index - offset
-        offset_item = self.ui.toolbar_layout_list.item(offset_index)
-        if offset_item:
-            current_item = self.ui.toolbar_layout_list.takeItem(current_index)
-            self.ui.toolbar_layout_list.insertItem(offset_index, current_item)
-            self.ui.toolbar_layout_list.setCurrentItem(current_item)
-            self.update_buttons()
-
     def remove_action(self):
         item = self.ui.toolbar_layout_list.takeItem(self.ui.toolbar_layout_list.currentRow())
         del item
@@ -271,34 +270,38 @@ class InterfaceOptionsPage(OptionsPage):
 
     def _update_toolbar(self):
         widget = self.parent()
-        while not isinstance(widget, QtGui.QMainWindow):
+        while not isinstance(widget, QtWidgets.QMainWindow):
             widget = widget.parent()
         # Call the main window's create toolbar method
         widget.create_action_toolbar()
         widget.set_tab_order()
 
 
-class ToolbarListItem(QtGui.QListWidgetItem):
+class ToolbarListItem(QtWidgets.QListWidgetItem):
     def __init__(self, action_name, *args, **kwargs):
-        super(ToolbarListItem, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.action_name = action_name
 
 
-class AddActionDialog(QtGui.QDialog):
+class AddActionDialog(QtWidgets.QDialog):
     def __init__(self, action_list, *args, **kwargs):
-        super(AddActionDialog, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
-        layout = QtGui.QVBoxLayout(self)
+        layout = QtWidgets.QVBoxLayout(self)
 
-        self.action_list = sorted([[_(self.parent().TOOLBAR_BUTTONS[action]['label']), action]
-                                  for action in action_list])
+        # TODO: Remove temporary workaround once https://github.com/python-babel/babel/issues/415 has been resolved.
+        babel_415_workaround_list = []
+        for action in action_list:
+            babel_415_workaround = self.parent().TOOLBAR_BUTTONS[action]['label']
+            babel_415_workaround_list.append([_(babel_415_workaround), action])
+        self.action_list = sorted(babel_415_workaround_list)
 
-        self.combo_box = QtGui.QComboBox(self)
+        self.combo_box = QtWidgets.QComboBox(self)
         self.combo_box.addItems([label for label, action in self.action_list])
         layout.addWidget(self.combo_box)
 
-        buttons = QtGui.QDialogButtonBox(
-            QtGui.QDialogButtonBox.Ok | QtGui.QDialogButtonBox.Cancel,
+        buttons = QtWidgets.QDialogButtonBox(
+            QtWidgets.QDialogButtonBox.Ok | QtWidgets.QDialogButtonBox.Cancel,
             QtCore.Qt.Horizontal, self)
         buttons.accepted.connect(self.accept)
         buttons.rejected.connect(self.reject)
@@ -312,7 +315,7 @@ class AddActionDialog(QtGui.QDialog):
         dialog = AddActionDialog(action_list, parent)
         result = dialog.exec_()
         selected_action = dialog.selected_action()
-        return (selected_action, result == QtGui.QDialog.Accepted)
+        return (selected_action, result == QtWidgets.QDialog.Accepted)
 
 
 register_options_page(InterfaceOptionsPage)
