@@ -17,13 +17,24 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-from PyQt4 import QtCore, QtGui
+from PyQt5 import (
+    QtCore,
+    QtGui,
+    QtWidgets,
+)
+
 from picard import config
 from picard.const import PICARD_URLS
 from picard.script import ScriptParser
-from picard.ui.options import OptionsPage, OptionsCheckError, register_options_page
-from picard.ui.ui_options_script import Ui_ScriptingOptionsPage
+from picard.util import restore_method
 
+from picard.ui import HashableListWidgetItem
+from picard.ui.options import (
+    OptionsCheckError,
+    OptionsPage,
+    register_options_page,
+)
+from picard.ui.ui_options_script import Ui_ScriptingOptionsPage
 
 DEFAULT_NUMBERED_SCRIPT_NAME = N_("My script %d")
 DEFAULT_SCRIPT_NAME = N_("My script")
@@ -32,7 +43,7 @@ DEFAULT_SCRIPT_NAME = N_("My script")
 class TaggerScriptSyntaxHighlighter(QtGui.QSyntaxHighlighter):
 
     def __init__(self, document):
-        QtGui.QSyntaxHighlighter.__init__(self, document)
+        super().__init__(document)
         self.func_re = QtCore.QRegExp(r"\$(?!noop)[a-zA-Z][_a-zA-Z0-9]*\(")
         self.func_fmt = QtGui.QTextCharFormat()
         self.func_fmt.setFontWeight(QtGui.QFont.Bold)
@@ -84,7 +95,7 @@ class TaggerScriptSyntaxHighlighter(QtGui.QSyntaxHighlighter):
             elif (next_index > -1) and text[next_index] == ')':
                 open_brackets -= 1
 
-            if (next_index > -1):
+            if next_index > -1:
                 self.setFormat(index, next_index - index + 1, self.noop_fmt)
             elif (next_index == -1) and (open_brackets > 0):
                 self.setFormat(index, len(text) - index, self.noop_fmt)
@@ -98,7 +109,7 @@ class TaggerScriptSyntaxHighlighter(QtGui.QSyntaxHighlighter):
         self.setCurrentBlockState(open_brackets)
 
 
-class AdvancedScriptItem(QtGui.QWidget):
+class AdvancedScriptItem(QtWidgets.QWidget):
     """Custom widget for script list items"""
 
     _CHECKBOX_POS = 0
@@ -108,37 +119,37 @@ class AdvancedScriptItem(QtGui.QWidget):
     _BUTTON_OTHER = 4
 
     def __init__(self, name=None, state=True, parent=None):
-        super(AdvancedScriptItem, self).__init__(parent)
-        layout = QtGui.QGridLayout()
+        super().__init__(parent)
+        layout = QtWidgets.QGridLayout()
         layout.setHorizontalSpacing(5)
         layout.setVerticalSpacing(2)
         layout.setContentsMargins(0, 0, 0, 0)
         self.setLayout(layout)
 
-        checkbox = QtGui.QCheckBox()
+        checkbox = QtWidgets.QCheckBox()
         checkbox.setChecked(state)
         checkbox.setMaximumSize(QtCore.QSize(22, 22))
         layout.addWidget(checkbox, 0, self._CHECKBOX_POS)
 
-        layout.addWidget(QtGui.QLabel(name), 0, self._NAME_POS)
+        layout.addWidget(QtWidgets.QLabel(name), 0, self._NAME_POS)
 
-        up_button = QtGui.QToolButton()
+        up_button = QtWidgets.QToolButton()
         up_button.setArrowType(QtCore.Qt.UpArrow)
         up_button.setMaximumSize(QtCore.QSize(16, 16))
         up_button.setToolTip(_("Move script up"))
-        down_button = QtGui.QToolButton()
+        down_button = QtWidgets.QToolButton()
         down_button.setArrowType(QtCore.Qt.DownArrow)
         down_button.setMaximumSize(QtCore.QSize(16, 16))
         down_button.setToolTip(_("Move script down"))
         layout.addWidget(up_button, 0, self._BUTTON_UP)
         layout.addWidget(down_button, 0, self._BUTTON_DOWN)
 
-        other_button = QtGui.QToolButton()
+        other_button = QtWidgets.QToolButton()
         other_button.setText("...")
         other_button.setAutoRaise(True)
         other_button.setMaximumSize(QtCore.QSize(16, 16))
         other_button.setToolTip(_("Other options"))
-        menu = QtGui.QMenu()
+        menu = QtWidgets.QMenu()
         menu.addAction(_("Rename script"))
         menu.addAction(_("Remove script"))
         self.menu = menu
@@ -185,6 +196,7 @@ class AdvancedScriptItem(QtGui.QWidget):
 
 class ScriptItem:
     """Holds a script's list and text widget properties and improves readability"""
+
     def __init__(self, pos, name=None, enabled=True, text=""):
         self.pos = pos
         if name is None:
@@ -215,7 +227,7 @@ class ScriptingOptionsPage(OptionsPage):
     ]
 
     def __init__(self, parent=None):
-        super(ScriptingOptionsPage, self).__init__(parent)
+        super().__init__(parent)
         self.ui = Ui_ScriptingOptionsPage()
         self.ui.setupUi(self)
         self.highlighter = TaggerScriptSyntaxHighlighter(self.ui.tagger_script.document())
@@ -230,6 +242,14 @@ class ScriptingOptionsPage(OptionsPage):
         self.last_selected_script_pos = 0
         self.ui.splitter.setStretchFactor(0, 1)
         self.ui.splitter.setStretchFactor(1, 2)
+        self.delete_shortcut = QtWidgets.QShortcut(self.ui.script_list)
+        self.delete_shortcut.setKey(QtGui.QKeySequence.Delete)
+        self.delete_shortcut.activated.connect(self.delete_selected_script)
+
+    def delete_selected_script(self):
+        items = self.ui.script_list.selectedItems()
+        if items:
+            self.remove_from_list_of_scripts(self.ui.script_list.row(items[0]))
 
     def script_name_changed(self):
         items = self.ui.script_list.selectedItems()
@@ -258,7 +278,7 @@ class ScriptingOptionsPage(OptionsPage):
         list_widget.set_rename_connection(lambda: self.rename_script(item))
 
     def rename_script(self, item):
-        self.ui.script_list.setItemSelected(item, True)
+        item.setSelected(True)
         self.ui.script_name.setFocus()
         self.ui.script_name.selectAll()
 
@@ -272,14 +292,14 @@ class ScriptingOptionsPage(OptionsPage):
         numbered_name = _(DEFAULT_NUMBERED_SCRIPT_NAME) % (count + 1)
         script = ScriptItem(pos=count, name=numbered_name)
 
-        list_item = QtGui.QListWidgetItem()
+        list_item = HashableListWidgetItem()
         list_widget = AdvancedScriptItem(numbered_name)
         self.setSignals(list_widget, list_item)
         self.ui.script_list.addItem(list_item)
         self.ui.script_list.setItemWidget(list_item, list_widget)
         self.listitem_to_scriptitem[list_item] = script
         self.list_of_scripts.append(script.get_all())
-        self.ui.script_list.setItemSelected(list_item, True)
+        list_item.setSelected(True)
 
     def update_script_positions(self):
         for i, script in enumerate(self.list_of_scripts):
@@ -289,11 +309,11 @@ class ScriptingOptionsPage(OptionsPage):
 
     def remove_from_list_of_scripts(self, row):
         item = self.ui.script_list.item(row)
-        confirm_remove = QtGui.QMessageBox()
+        confirm_remove = QtWidgets.QMessageBox()
         msg = _("Are you sure you want to remove this script?")
-        reply = confirm_remove.question(confirm_remove, _('Confirm Remove'), msg, QtGui.QMessageBox.Yes,
-                                        QtGui.QMessageBox.No)
-        if item and reply == QtGui.QMessageBox.Yes:
+        reply = confirm_remove.question(confirm_remove, _('Confirm Remove'), msg, QtWidgets.QMessageBox.Yes,
+                                        QtWidgets.QMessageBox.No)
+        if item and reply == QtWidgets.QMessageBox.Yes:
             item = self.ui.script_list.takeItem(row)
             script = self.listitem_to_scriptitem[item]
             del self.listitem_to_scriptitem[item]
@@ -315,10 +335,11 @@ class ScriptingOptionsPage(OptionsPage):
                 if not self.ui.script_list.selectedItems():
                     current_item = self.ui.script_list.currentItem()
                     if current_item:
-                        self.ui.script_list.setItemSelected(current_item, True)
+                        current_item.setSelected(True)
                     else:
                         item = self.ui.script_list.item(0)
-                        self.ui.script_list.setItemSelected(item, True)
+                        if item:
+                            item.setSelected(True)
             elif row < self.last_selected_script_pos:
                 self.last_selected_script_pos -= 1
 
@@ -372,7 +393,7 @@ class ScriptingOptionsPage(OptionsPage):
     def check(self):
         parser = ScriptParser()
         try:
-            parser.eval(unicode(self.ui.tagger_script.toPlainText()))
+            parser.eval(self.ui.tagger_script.toPlainText())
         except Exception as e:
             raise OptionsCheckError(_("Script Error"), str(e))
 
@@ -381,14 +402,14 @@ class ScriptingOptionsPage(OptionsPage):
         self.ui.script_list.clear()
         self.ui.script_name.setText("")
         self.ui.tagger_script.setText("")
-        super(ScriptingOptionsPage, self).restore_defaults()
+        super().restore_defaults()
 
     def load(self):
         self.ui.enable_tagger_scripts.setChecked(config.setting["enable_tagger_scripts"])
         self.list_of_scripts = config.setting["list_of_scripts"]
         for s_pos, s_name, s_enabled, s_text in self.list_of_scripts:
             script = ScriptItem(s_pos, s_name, s_enabled, s_text)
-            list_item = QtGui.QListWidgetItem()
+            list_item = HashableListWidgetItem()
             list_widget = AdvancedScriptItem(name=s_name, state=s_enabled)
             self.setSignals(list_widget, list_item)
             self.ui.script_list.addItem(list_item)
@@ -399,18 +420,22 @@ class ScriptingOptionsPage(OptionsPage):
         self.last_selected_script_pos = config.persist["last_selected_script_pos"]
         last_selected_script = self.ui.script_list.item(self.last_selected_script_pos)
         if last_selected_script:
-            self.ui.script_list.setItemSelected(last_selected_script, True)
+            last_selected_script.setSelected(True)
 
-        # Preserve previous splitter position
-        self.ui.splitter.restoreState(config.persist["scripting_splitter"])
+        self.restore_state()
 
         args = {
             "picard-doc-scripting-url": PICARD_URLS['doc_scripting'],
         }
-        text = _(u'<a href="%(picard-doc-scripting-url)s">Open Scripting'
+        text = _('<a href="%(picard-doc-scripting-url)s">Open Scripting'
                  ' Documentation in your browser</a>') % args
         self.ui.scripting_doc_link.setText(text)
 
+    @restore_method
+    def restore_state(self):
+        # Preserve previous splitter position
+        self.ui.splitter.restoreState(config.persist["scripting_splitter"])
+
     def save(self):
         config.setting["enable_tagger_scripts"] = self.ui.enable_tagger_scripts.isChecked()
         config.setting["list_of_scripts"] = self.list_of_scripts
