@@ -18,8 +18,8 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 
-from PyQt4 import QtCore
 from picard import config
+
 from picard.ui import PicardDialog
 from picard.ui.ui_passworddialog import Ui_PasswordDialog
 
@@ -27,7 +27,7 @@ from picard.ui.ui_passworddialog import Ui_PasswordDialog
 class PasswordDialog(PicardDialog):
 
     def __init__(self, authenticator, reply, parent=None):
-        PicardDialog.__init__(self, parent)
+        super().__init__(parent)
         self._authenticator = authenticator
         self.ui = Ui_PasswordDialog()
         self.ui.setupUi(self)
@@ -39,15 +39,15 @@ class PasswordDialog(PicardDialog):
         self.ui.buttonbox.accepted.connect(self.set_new_password)
 
     def set_new_password(self):
-        self._authenticator.setUser(unicode(self.ui.username.text()))
-        self._authenticator.setPassword(unicode(self.ui.password.text()))
+        self._authenticator.setUser(self.ui.username.text())
+        self._authenticator.setPassword(self.ui.password.text())
         self.accept()
 
 
 class ProxyDialog(PicardDialog):
 
     def __init__(self, authenticator, proxy, parent=None):
-        PicardDialog.__init__(self, parent)
+        super().__init__(parent)
         self._authenticator = authenticator
         self._proxy = proxy
         self.ui = Ui_PasswordDialog()
@@ -59,8 +59,8 @@ class ProxyDialog(PicardDialog):
         self.ui.buttonbox.accepted.connect(self.set_proxy_password)
 
     def set_proxy_password(self):
-        config.setting["proxy_username"] = unicode(self.ui.username.text())
-        config.setting["proxy_password"] = unicode(self.ui.password.text())
-        self._authenticator.setUser(unicode(self.ui.username.text()))
-        self._authenticator.setPassword(unicode(self.ui.password.text()))
+        config.setting["proxy_username"] = self.ui.username.text()
+        config.setting["proxy_password"] = self.ui.password.text()
+        self._authenticator.setUser(self.ui.username.text())
+        self._authenticator.setPassword(self.ui.password.text())
         self.accept()
