@@ -1,9 +1,13 @@
 #!/usr/bin/env python
 
-import os.path
 from distutils import log
 from distutils.dep_util import newer
-from distutils.spawn import find_executable, spawn, DistutilsExecError
+from distutils.spawn import (
+    DistutilsExecError,
+    find_executable,
+    spawn,
+)
+import os.path
 
 
 def main():
@@ -12,7 +16,7 @@ def main():
     pyfile = os.path.join(topdir, "picard", "resources.py")
     qrcfile = os.path.join(topdir, "resources", "picard.qrc")
     if newer(qrcfile, pyfile):
-        pyrcc = 'pyrcc4'
+        pyrcc = 'pyrcc5'
         pyrcc_path = find_executable(pyrcc)
         if pyrcc_path is None:
             log.error("%s command not found, cannot build resource file !", pyrcc)
