@@ -1,9 +1,11 @@
 #!/usr/bin/env python
 
-import os, fnmatch
-import re
 from distutils import log
 from distutils.dep_util import newer
+import fnmatch
+import os
+import re
+
 
 """Build a Qt resources file with all png images found under images/
 It will update qrc file only if images newer than it are found
@@ -46,10 +48,10 @@ def main():
     if newimages:
         log.info("%d images newer than %s found" % (newimages, qrcfile))
         with open(qrcfile, 'wb+') as f:
-            f.write('<!DOCTYPE RCC><RCC version="1.0">\n<qresource>\n')
+            f.write(b'<!DOCTYPE RCC><RCC version="1.0">\n<qresource>\n')
             for filename in sorted(images, key=natsort_key):
-                f.write('    <file>%s</file>\n' % filename.replace('\\', '/'))
-            f.write('</qresource>\n</RCC>\n')
+                f.write(('    <file>%s</file>\n' % filename.replace('\\', '/')).encode())
+            f.write(b'</qresource>\n</RCC>\n')
             log.info("File %s written, %d images" % (qrcfile, len(images)))
 
 
