@@ -1,16 +1,26 @@
 # -*- coding: utf-8 -*-
+import logging
 import os.path
-import picard.formats
-import unittest
 import shutil
+from tempfile import mkstemp
+from test.picardtestcase import PicardTestCase
+import unittest
+
+import mutagen
 
 from PyQt5 import QtCore
-from picard import config, log
-from picard.coverart.image import CoverArtImage, TagCoverArtImage
-from picard.metadata import Metadata
-from picard.formats import ext_to_format
-from tempfile import mkstemp
 
+from picard import (
+    config,
+    log,
+)
+from picard.coverart.image import (
+    CoverArtImage,
+    TagCoverArtImage,
+)
+import picard.formats
+from picard.formats import ext_to_format
+from picard.metadata import Metadata
 
 settings = {
     'clear_existing_tags': False,
@@ -26,30 +36,18 @@ settings = {
     'save_images_to_tags': True,
     'write_id3v1': True,
     'write_id3v23': False,
+    'itunes_compatible_grouping': False,
 }
 
 
-class FakeTagger(QtCore.QObject):
+def save_metadata(filename, metadata):
+    f = picard.formats.open_(filename)
+    f._save(filename, metadata)
 
-    tagger_stats_changed = QtCore.pyqtSignal()
 
-    def __init__(self):
-        QtCore.QObject.__init__(self)
-        QtCore.QObject.config = config
-        QtCore.QObject.log = log
-        self.tagger_stats_changed.connect(self.emit)
-        self.exit_cleanup = []
-        self.files = {}
-
-    def register_cleanup(self, func):
-        self.exit_cleanup.append(func)
-
-    def run_cleanup(self):
-        for f in self.exit_cleanup:
-            f()
-
-    def emit(self, *args):
-        pass
+def load_metadata(filename):
+    f = picard.formats.open_(filename)
+    return f._load(filename)
 
 
 def save_and_load_metadata(filename, metadata):
@@ -58,10 +56,11 @@ def save_and_load_metadata(filename, metadata):
     loaded_metadata = f._load(filename)
     f._copy_loaded_metadata(loaded_metadata)
     f._save(filename, metadata)
-    f = picard.formats.open_(filename)
-    loaded_metadata = f._load(filename)
+    loaded_metadata = load_metadata(filename)
     return loaded_metadata
 
+def load_raw(filename):
+    return mutagen.File(filename)
 
 TAGS = {
     'albumartist': 'Foo',
@@ -100,24 +99,33 @@ TAGS = {
     'media': 'Foo',
     'mixer': 'Foo',
     'mood': 'Foo',
-    'musicbrainz_albumartistid': 'Foo',
-    'musicbrainz_albumid': 'Foo',
-    'musicbrainz_artistid': 'Foo',
-    'musicbrainz_discid': 'Foo',
-    'musicbrainz_trackid': 'Foo',
+    'movement': 'Foo',
+    'movementnumber': '2',
+    'movementtotal': '8',
+    'musicbrainz_albumartistid': '00000000-0000-0000-0000-000000000000',
+    'musicbrainz_albumid': '00000000-0000-0000-0000-000000000000',
+    'musicbrainz_artistid': '00000000-0000-0000-0000-000000000000',
+    'musicbrainz_discid': 'HJRFvVfxx0MU_6v8v9swQUxDmZQ-',
+    'musicbrainz_originalalbumid': '00000000-0000-0000-0000-000000000000',
+    'musicbrainz_originalartistid': '00000000-0000-0000-0000-000000000000',
+    'musicbrainz_releasegroupid': '00000000-0000-0000-0000-000000000000',
+    'musicbrainz_trackid': '00000000-0000-0000-0000-000000000000',
     'musicbrainz_trmid': 'Foo',
+    'musicbrainz_workid': '00000000-0000-0000-0000-000000000000',
     'musicip_fingerprint': 'Foo',
-    'musicip_puid': 'Foo',
+    'musicip_puid': '00000000-0000-0000-0000-000000000000',
     'originaldate': '1980-01-20',
     'originalyear': '1980',
     'performer:guest vocal': 'Foo',
     'podcast': '1',
     'podcasturl': 'Foo',
     'producer': 'Foo',
+    'releasecountry': 'XW',
     'releasestatus': 'Foo',
     'releasetype': 'Foo',
     'remixer': 'Foo',
     'show': 'Foo',
+    'showmovement': '1',
     'showsort': 'Foo',
     'subtitle': 'Foo',
     'title': 'Foo',
@@ -126,6 +134,7 @@ TAGS = {
     'totaltracks': '10',
     'tracknumber': '2',
     'website': 'http://example.com',
+    'work': 'Foo'
 }
 
 
@@ -140,17 +149,17 @@ def skipUnlessTestfile(func):
 # prevent unittest to run tests in those classes
 class CommonTests:
 
-    class FormatsTest(unittest.TestCase):
+    class FormatsTest(PicardTestCase):
 
         testfile = None
         testfile_ext = None
         testfile_path = None
 
         def setUp(self):
+            super().setUp()
             self.tags = TAGS.copy()
             _name, self.testfile_ext = os.path.splitext(self.testfile)
             config.setting = settings.copy()
-            QtCore.QObject.tagger = FakeTagger()
             if self.testfile:
                 self.testfile_path = os.path.join('test', 'data', self.testfile)
                 self.testfile_ext = os.path.splitext(self.testfile)[1]
@@ -158,15 +167,19 @@ class CommonTests:
             self.setup_tags()
 
         def copy_of_original_testfile(self):
-            fd, copy = mkstemp(suffix=self.testfile_ext)
+            return self.copy_file_tmp(self.testfile_path, self.testfile_ext)
+
+        def copy_file_tmp(self, filename, ext):
+            fd, copy = mkstemp(suffix=ext)
             self.addCleanup(os.unlink, copy)
             os.close(fd)
-            shutil.copy(self.testfile_path, copy)
+            shutil.copy(filename, copy)
             return copy
 
         def setup_tags(self):
             supports_tag = ext_to_format(self.testfile_ext[1:]).supports_tag
-            self.remove_tags([tag for tag in self.tags if not supports_tag(tag)])
+            self.unsupported_tags = {tag : val for tag, val in self.tags.items() if not supports_tag(tag)}
+            self.remove_tags(self.unsupported_tags.keys())
 
         def set_tags(self, dict_tag_value=None):
             if dict_tag_value:
@@ -185,6 +198,25 @@ class CommonTests:
             for (key, value) in self.tags.items():
                 self.assertEqual(loaded_metadata[key], value, '%s: %r != %r' % (key, loaded_metadata[key], value))
 
+        @skipUnlessTestfile
+        def test_unsupported_tags(self):
+            metadata = Metadata()
+            for (key, value) in self.unsupported_tags.items():
+                metadata[key] = value
+            loaded_metadata = save_and_load_metadata(self.filename, metadata)
+            for tag in self.unsupported_tags:
+                self.assertTrue(tag not in loaded_metadata, '%s: %r != None' % (tag, loaded_metadata[tag]))
+
+        @skipUnlessTestfile
+        def test_preserve_unchanged_tags(self):
+            metadata = Metadata()
+            for (key, value) in self.tags.items():
+                metadata[key] = value
+            save_metadata(self.filename, metadata)
+            loaded_metadata = save_and_load_metadata(self.filename, Metadata())
+            for (key, value) in self.tags.items():
+                self.assertEqual(loaded_metadata[key], value, '%s: %r != %r' % (key, loaded_metadata[key], value))
+
         @skipUnlessTestfile
         def test_delete_simple_tags(self):
             metadata = Metadata()
@@ -203,6 +235,17 @@ class CommonTests:
                 self.assertIn('~rating', original_metadata.keys())
                 self.assertNotIn('~rating', new_metadata.keys())
 
+        @skipUnlessTestfile
+        def test_delete_non_existant_tags(self):
+            metadata = Metadata()
+            metadata.delete('albumartist')
+            metadata.delete('performer:drums')
+            metadata.delete('totaltracks')
+            new_metadata = save_and_load_metadata(self.filename, metadata)
+            self.assertNotIn('albumartist', new_metadata.keys())
+            self.assertNotIn('performer:drums', new_metadata.keys())
+            self.assertNotIn('totaltracks', new_metadata.keys())
+
         @skipUnlessTestfile
         def test_delete_complex_tags(self):
             metadata = Metadata()
@@ -264,7 +307,6 @@ class CommonTests:
             self.assertEqual(f._fixed_splitext('.test'), os.path.splitext('.test'))
             self.assertNotEqual(f._fixed_splitext(f.EXTENSIONS[0]), os.path.splitext(f.EXTENSIONS[0]))
 
-
     class ID3Test(FormatsTest):
 
         def setup_tags(self):
@@ -325,10 +367,6 @@ class CommonTests:
         @skipUnlessTestfile
         def test_performer_duplication(self):
 
-            def reset_id3_ver():
-                config.setting['write_id3v23'] = False
-
-            self.addCleanup(reset_id3_ver)
             config.setting['write_id3v23'] = True
             metadata = Metadata()
             tags = {
@@ -363,11 +401,7 @@ class CommonTests:
 
         @skipUnlessTestfile
         def test_id3v23_simple_tags(self):
-
-            def reset_to_id3v24():
-                config.setting['write_id3v23'] = False
             config.setting['write_id3v23'] = True
-            self.addCleanup(reset_to_id3v24)
             metadata = Metadata()
             for (key, value) in self.tags.items():
                 metadata[key] = value
@@ -375,17 +409,115 @@ class CommonTests:
             for (key, value) in self.tags.items():
                 self.assertEqual(loaded_metadata[key], value, '%s: %r != %r' % (key, loaded_metadata[key], value))
 
+        @property
+        def itunes_grouping_metadata(self):
+            metadata = Metadata()
+            metadata['grouping'] = 'The Grouping'
+            metadata['work'] = 'The Work'
+            return metadata
+
+        @skipUnlessTestfile
+        def test_standard_grouping(self):
+            metadata = self.itunes_grouping_metadata
+
+            config.setting['itunes_compatible_grouping'] = False
+            loaded_metadata = save_and_load_metadata(self.filename, metadata)
+
+            self.assertEqual(loaded_metadata['grouping'], metadata['grouping'])
+            self.assertEqual(loaded_metadata['work'], metadata['work'])
+
+        @skipUnlessTestfile
+        def test_itunes_compatible_grouping(self):
+            metadata = self.itunes_grouping_metadata
+
+            config.setting['itunes_compatible_grouping'] = True
+            loaded_metadata = save_and_load_metadata(self.filename, metadata)
+
+            self.assertEqual(loaded_metadata['grouping'], metadata['grouping'])
+            self.assertEqual(loaded_metadata['work'], metadata['work'])
+
+        @skipUnlessTestfile
+        def test_always_read_grp1(self):
+            metadata = self.itunes_grouping_metadata
+
+            config.setting['itunes_compatible_grouping'] = True
+            save_metadata(self.filename, metadata)
+            config.setting['itunes_compatible_grouping'] = False
+            loaded_metadata = load_metadata(self.filename)
+
+            self.assertIn(metadata['grouping'], loaded_metadata['grouping'])
+            self.assertIn(metadata['work'], loaded_metadata['grouping'])
+            self.assertEqual(loaded_metadata['work'], '')
+
+        @skipUnlessTestfile
+        def test_always_read_txxx_work(self):
+            metadata = self.itunes_grouping_metadata
+
+            config.setting['itunes_compatible_grouping'] = False
+            save_metadata(self.filename, metadata)
+            config.setting['itunes_compatible_grouping'] = True
+            loaded_metadata = load_metadata(self.filename)
+
+            self.assertIn(metadata['grouping'], loaded_metadata['work'])
+            self.assertIn(metadata['work'], loaded_metadata['work'])
+            self.assertEqual(loaded_metadata['grouping'], '')
+
+        @skipUnlessTestfile
+        def test_save_itunnorm_tag(self):
+            config.setting['clear_existing_tags'] = True
+            iTunNORM = '00001E86 00001E86 0000A2A3 0000A2A3 000006A6 000006A6 000078FA 000078FA 00000211 00000211'
+            metadata = Metadata()
+            metadata['comment:iTunNORM'] = iTunNORM
+            new_metadata = save_and_load_metadata(self.filename, metadata)
+            self.assertEqual(new_metadata['comment:iTunNORM'], iTunNORM)
+
+        def test_rename_txxx_tags(self):
+            file_path = os.path.join('test', 'data', 'test-id3-rename-tags.mp3')
+            filename = self.copy_file_tmp(file_path, 'mp3')
+            raw_metadata = load_raw(filename)
+            self.assertTrue('TXXX:Artists' in raw_metadata)
+            self.assertFalse('TXXX:ARTISTS' in raw_metadata)
+            self.assertTrue('TXXX:Work' in raw_metadata)
+            self.assertFalse('TXXX:WORK' in raw_metadata)
+            metadata = load_metadata(filename)
+            self.assertEqual(metadata['artists'], 'Artist1; Artist2')
+            self.assertFalse('Artists' in metadata)
+            self.assertEqual(metadata['work'], 'The Work')
+            self.assertFalse('Work' in metadata)
+            save_metadata(filename, metadata)
+            raw_metadata = load_raw(filename)
+            self.assertFalse('TXXX:Artists' in raw_metadata)
+            self.assertTrue('TXXX:ARTISTS' in raw_metadata)
+            self.assertFalse('TXXX:Work' in raw_metadata)
+            self.assertTrue('TXXX:WORK' in raw_metadata)
+
+        def test_preserve_unchanged_tags_v23(self):
+            config.setting['write_id3v23'] = True
+            self.test_preserve_unchanged_tags()
+
 
 class FLACTest(CommonTests.FormatsTest):
     testfile = 'test.flac'
     supports_ratings = True
 
+    def test_preserve_waveformatextensible_channel_mask(self):
+        config.setting['clear_existing_tags'] = True
+        original_metadata = load_metadata(self.filename)
+        self.assertEqual(original_metadata['~waveformatextensible_channel_mask'], '0x3')
+        new_metadata = save_and_load_metadata(self.filename, original_metadata)
+        self.assertEqual(new_metadata['~waveformatextensible_channel_mask'], '0x3')
+
 
 class WMATest(CommonTests.FormatsTest):
     testfile = 'test.wma'
     supports_ratings = True
 
 
+class ASFTest(CommonTests.FormatsTest):
+    testfile = 'test.asf'
+    supports_ratings = True
+
+
 class MP3Test(CommonTests.ID3Test):
     testfile = 'test.mp3'
     supports_ratings = True
@@ -401,21 +533,48 @@ class DSFTest(CommonTests.ID3Test):
     supports_ratings = True
 
 
-if picard.formats.AiffFile:
-    class AIFFTest(CommonTests.ID3Test):
-        testfile = 'test.aiff'
-        supports_ratings = False
+class AIFFTest(CommonTests.ID3Test):
+    testfile = 'test.aiff'
+    supports_ratings = False
 
 
 class OggVorbisTest(CommonTests.FormatsTest):
     testfile = 'test.ogg'
     supports_ratings = True
 
+    def test_invalid_rating(self):
+        filename = os.path.join('test', 'data', 'test-invalid-rating.ogg')
+        old_log_level = log.get_effective_level()
+        log.set_level(logging.ERROR)
+        metadata = load_metadata(filename)
+        log.set_level(old_log_level)
+        self.assertEqual(metadata["~rating"], "THERATING")
+
+
+class OggSpxTest(CommonTests.FormatsTest):
+    testfile = 'test.spx'
+    supports_ratings = True
+
+
+class OggOpusTest(CommonTests.FormatsTest):
+    testfile = 'test.spx'
+    supports_ratings = True
+
 
 class MP4Test(CommonTests.FormatsTest):
     testfile = 'test.m4a'
     supports_ratings = False
 
+    def test_supports_tag(self):
+        fmt = ext_to_format(self.testfile_ext[1:])
+        self.assertTrue(fmt.supports_tag('copyright'))
+        self.assertTrue(fmt.supports_tag('compilation'))
+        self.assertTrue(fmt.supports_tag('bpm'))
+        self.assertTrue(fmt.supports_tag('djmixer'))
+        self.assertTrue(fmt.supports_tag('discnumber'))
+        self.assertTrue(fmt.supports_tag('lyrics:lead'))
+        self.assertTrue(fmt.supports_tag('~length'))
+
 
 class WavPackTest(CommonTests.FormatsTest):
     testfile = 'test.wv'
@@ -432,14 +591,43 @@ class MusepackSV8Test(CommonTests.FormatsTest):
     supports_ratings = False
 
 
+class MonkeysAudioTest(CommonTests.FormatsTest):
+    testfile = 'test.ape'
+    supports_ratings = False
+
+
+class TAKTest(CommonTests.FormatsTest):
+    testfile = 'test.tak'
+    supports_ratings = False
+
+
+class OptimFROGLosslessTest(CommonTests.FormatsTest):
+    testfile = 'test.ofr'
+    supports_ratings = False
+
+    def test_format(self):
+        metadata = load_metadata(self.filename)
+        self.assertEqual(metadata['~format'], 'OptimFROG Lossless Audio')
+
+
+class OptimFROGDUalStreamTest(CommonTests.FormatsTest):
+    testfile = 'test.ofs'
+    supports_ratings = False
+
+    def test_format(self):
+        metadata = load_metadata(self.filename)
+        self.assertEqual(metadata['~format'], 'OptimFROG DualStream Audio')
+
+
 cover_settings = {
     'embed_only_one_front_image': True,
 }
 
 
-class TestCoverArt(unittest.TestCase):
+class TestCoverArt(PicardTestCase):
 
     def setUp(self):
+        super().setUp()
         with open(os.path.join('test', 'data', 'mb.jpg'), 'rb') as f:
             self.jpegdata = f.read()
         with open(os.path.join('test', 'data', 'mb.png'), 'rb') as f:
@@ -449,7 +637,6 @@ class TestCoverArt(unittest.TestCase):
         config.setting = settings.copy()
         if extra is not None:
             config.setting.update(extra)
-        QtCore.QObject.tagger = FakeTagger()
 
     def _set_up(self, original, extra=None):
         fd, self.filename = mkstemp(suffix=os.path.splitext(original)[1])
@@ -720,8 +907,41 @@ class TestCoverArt(unittest.TestCase):
             self._tear_down()
 
 
-class WAVTest(unittest.TestCase):
+class AACTest(PicardTestCase):
+    filename = os.path.join('test', 'data', 'test.aac')
+
+    def setUp(self):
+        super().setUp()
+        config.setting = settings.copy()
+
+    def test_can_open_and_save(self):
+        metadata = Metadata()
+        save_and_load_metadata(self.filename, metadata)
+
+
+class AACWithAPETest(CommonTests.FormatsTest):
+    testfile = 'test-apev2.aac'
+    supports_ratings = False
+
+
+class WAVTest(PicardTestCase):
     filename = os.path.join('test', 'data', 'test.wav')
+
+    def setUp(self):
+        super().setUp()
+        config.setting = settings.copy()
+
+    def test_can_open_and_save(self):
+        metadata = Metadata()
+        save_and_load_metadata(self.filename, metadata)
+
+
+class MIDITest(PicardTestCase):
+    filename = os.path.join('test', 'data', 'test.mid')
+
+    def setUp(self):
+        super().setUp()
+        config.setting = settings.copy()
 
     def test_can_open_and_save(self):
         metadata = Metadata()
