@@ -1,7 +1,8 @@
 # -*- coding: utf-8 -*-
+from test.picardtestcase import PicardTestCase
 
-import unittest
 from picard import util
+
 #from picard.util import textencoding
 
 # Set the value to true below to show the coverage of Latin characters
@@ -104,7 +105,7 @@ combinations_to = (
 ascii_chars = u" !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~"
 
 
-class CompatibilityTest(unittest.TestCase):
+class CompatibilityTest(PicardTestCase):
 
     def test_correct(self):
         self.maxDiff = None
@@ -117,7 +118,7 @@ class CompatibilityTest(unittest.TestCase):
         pass
 
 
-class PunctuationTest(unittest.TestCase):
+class PunctuationTest(PicardTestCase):
 
     def test_correct(self):
         self.maxDiff = None
@@ -134,7 +135,7 @@ class PunctuationTest(unittest.TestCase):
         pass
 
 
-class CombinationsTest(unittest.TestCase):
+class CombinationsTest(PicardTestCase):
 
     def test_correct(self):
         self.maxDiff = None
@@ -151,7 +152,7 @@ class CombinationsTest(unittest.TestCase):
         pass
 
 
-class AsciiPunctTest(unittest.TestCase):
+class AsciiPunctTest(PicardTestCase):
 
     def test_correct(self):
         self.assertEqual(util.textencoding.asciipunct(u"‘Test’"), u"'Test'") # Quotations
@@ -163,7 +164,7 @@ class AsciiPunctTest(unittest.TestCase):
         pass
 
 
-class UnaccentTest(unittest.TestCase):
+class UnaccentTest(PicardTestCase):
 
     def test_correct(self):
         self.assertEqual(util.textencoding.unaccent(u"Lukáš"), u"Lukas")
@@ -178,7 +179,7 @@ class UnaccentTest(unittest.TestCase):
         self.assertNotEqual(util.textencoding.unaccent(u"ænima"), u"aenima")
 
 
-class ReplaceNonAsciiTest(unittest.TestCase):
+class ReplaceNonAsciiTest(PicardTestCase):
 
     def test_correct(self):
         self.assertEqual(util.textencoding.replace_non_ascii(u"Lukáš"), u"Lukas")
@@ -241,4 +242,3 @@ if show_latin2ascii_coverage:
     print("letter-like:   ",util.textencoding.replace_non_ascii(letter_like))
     print("enclosed:      ",util.textencoding.replace_non_ascii(enclosed))
     print()
-
