@@ -2,21 +2,24 @@
 
 import os
 import os.path
-import unittest
 import sys
+from test.picardtestcase import PicardTestCase
+import unittest
+
 from picard.util.filenaming import make_short_filename
 
 
-class ShortFilenameTest(unittest.TestCase):
+class ShortFilenameTest(PicardTestCase):
 
     def __init__(self, *args, **kwargs):
         self.maxDiff = None
-        self.root = os.path.join(sys.platform == "win32" and "X:\\" or "/", "x" * 10)
-        if sys.platform in ("win32"):
+        is_win32 = sys.platform == "win32"
+        self.root = os.path.join(is_win32 and "X:\\" or "/", "x" * 10)
+        if is_win32:
             self.max_len = 255
         else:
             self.max_len = os.statvfs("/").f_namemax
-        super(ShortFilenameTest, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     @unittest.skipUnless(sys.platform in ("win32", "darwin"), "windows / os x test")
     def test_bmp_unicode_on_unicode_fs(self):
@@ -79,7 +82,7 @@ class ShortFilenameTest(unittest.TestCase):
         root = os.path.join(self.root, "w" * 10, "x" * 10, "y" * 9, "z" * 9)
         fn = make_short_filename(
             root, os.path.join("b" * 200, "c" * 200, "d" * 200 + ".ext"),
-            win_compat=True, relative_to = self.root)
+            win_compat=True, relative_to=self.root)
         self.assertEqual(fn, os.path.join("b" * 100, "c" * 100, "d" * 7 + ".ext"))
 
     def test_windows_node_maxlength_shortening(self):
