@@ -1,17 +1,20 @@
 # -*- coding: utf-8 -*-
 
+import builtins
 import os.path
 import sys
+from test.picardtestcase import PicardTestCase
 import unittest
+
 from picard import util
+from picard.util import imageinfo
 
-import __builtin__
 # ensure _() is defined
-if '_' not in __builtin__.__dict__:
-    __builtin__.__dict__['_'] = lambda a: a
+if '_' not in builtins.__dict__:
+    builtins.__dict__['_'] = lambda a: a
 
 
-class ReplaceWin32IncompatTest(unittest.TestCase):
+class ReplaceWin32IncompatTest(PicardTestCase):
 
     @unittest.skipUnless(sys.platform == "win32", "windows test")
     def test_correct_absolute_win32(self):
@@ -38,7 +41,7 @@ class ReplaceWin32IncompatTest(unittest.TestCase):
                              "c:\\test\\te\"st2")
 
 
-class SanitizeDateTest(unittest.TestCase):
+class SanitizeDateTest(PicardTestCase):
 
     def test_correct(self):
         self.assertEqual(util.sanitize_date("2006--"), "2006")
@@ -53,7 +56,7 @@ class SanitizeDateTest(unittest.TestCase):
         self.assertNotEqual(util.sanitize_date("2006.03.02"), "2006-03-02")
 
 
-class TranslateArtistTest(unittest.TestCase):
+class TranslateArtistTest(PicardTestCase):
 
     def test_latin(self):
         self.assertEqual(u"Jean Michel Jarre", util.translate_from_sortname(u"Jean Michel Jarre", u"Jarre, Jean Michel"))
@@ -75,7 +78,7 @@ class TranslateArtistTest(unittest.TestCase):
         self.assertNotEqual(u"Пётр Ильич Чайковский", util.translate_from_sortname(u"Пётр Ильич Чайковский", u"Tchaikovsky, Pyotr Ilyich"))
 
 
-class FormatTimeTest(unittest.TestCase):
+class FormatTimeTest(PicardTestCase):
 
     def test(self):
         self.assertEqual("?:??", util.format_time(0))
@@ -84,7 +87,7 @@ class FormatTimeTest(unittest.TestCase):
         self.assertEqual("2:59", util.format_time(179499))
 
 
-class HiddenFileTest(unittest.TestCase):
+class HiddenFileTest(PicardTestCase):
 
     @unittest.skipUnless(sys.platform != "win32", "non-windows test")
     def test(self):
@@ -93,7 +96,7 @@ class HiddenFileTest(unittest.TestCase):
         self.assertFalse(util.is_hidden('/a/.b/c.mp3'))
 
 
-class TagsTest(unittest.TestCase):
+class TagsTest(PicardTestCase):
 
     def test_display_tag_name(self):
         dtn = util.tags.display_tag_name
@@ -107,7 +110,7 @@ class TagsTest(unittest.TestCase):
         self.assertEqual(dtn(''), '')
 
 
-class LinearCombinationTest(unittest.TestCase):
+class LinearCombinationTest(PicardTestCase):
 
     def test_0(self):
         parts = []
@@ -150,11 +153,10 @@ class LinearCombinationTest(unittest.TestCase):
         self.assertRaises(TypeError, util.linear_combination_of_weights, parts)
 
 
-class AlbumArtistFromPathTest(unittest.TestCase):
+class AlbumArtistFromPathTest(PicardTestCase):
 
     def test_album_artist_from_path(self):
         aafp = util.album_artist_from_path
-        from picard.file import File
         file_1 = r"/10cc/Original Soundtrack/02 I'm Not in Love.mp3"
         file_2 = r"/10cc - Original Soundtrack/02 I'm Not in Love.mp3"
         file_3 = r"/Original Soundtrack/02 I'm Not in Love.mp3"
@@ -177,10 +179,7 @@ class AlbumArtistFromPathTest(unittest.TestCase):
         self.assertEqual(aafp(file_4, 'album', 'artist'), ('album', 'artist'))
 
 
-from picard.util import imageinfo
-
-
-class ImageInfoTest(unittest.TestCase):
+class ImageInfoTest(PicardTestCase):
 
     def test_gif(self):
         file = os.path.join('test', 'data', 'mb.gif')
@@ -197,7 +196,7 @@ class ImageInfoTest(unittest.TestCase):
         with open(file, 'rb') as f:
             self.assertEqual(
                 imageinfo.identify(f.read()),
-                (140, 96, 'image/png', '.png', 11424)
+                (140, 96, 'image/png', '.png', 11137)
             )
 
     def test_jpeg(self):
@@ -226,3 +225,21 @@ class ImageInfoTest(unittest.TestCase):
                           imageinfo.identify, data)
         self.assertRaises(imageinfo.UnrecognizedFormat,
                           imageinfo.identify, data)
+
+class CompareBarcodesTest(unittest.TestCase):
+
+    def test_same(self):
+        self.assertTrue(util.compare_barcodes('0727361379704', '0727361379704'))
+        self.assertTrue(util.compare_barcodes('727361379704', '727361379704'))
+        self.assertTrue(util.compare_barcodes('727361379704', '0727361379704'))
+        self.assertTrue(util.compare_barcodes('0727361379704', '727361379704'))
+        self.assertTrue(util.compare_barcodes(None, None))
+        self.assertTrue(util.compare_barcodes('', ''))
+        self.assertTrue(util.compare_barcodes(None, ''))
+        self.assertTrue(util.compare_barcodes('', None))
+
+    def test_not_same(self):
+        self.assertFalse(util.compare_barcodes('0727361379704', '0727361379705'))
+        self.assertFalse(util.compare_barcodes('727361379704', '1727361379704'))
+        self.assertFalse(util.compare_barcodes('0727361379704', None))
+        self.assertFalse(util.compare_barcodes(None, '0727361379704'))
